-- This is a sample Musubi configuration file in lua format 
-- Its main purpose is to show the options you might want to set for your
-- simulation. It thus serves as a little documentation for the configuration.
-- 
-- WARNING!!!
-- LEAVE THIS FILE UNCHANGED IN THE REPOSITORY
-- unless you change configurable options, in this case you should update this
-- script accordingly.
--
-- This script should always serve as an example which runs "out of the box".
-- 
-- Thank you!
--------------------------------------------------------------------------------

-- With the simulation name you can put a label on your simulation run, to have
-- some concise reference. The name is arbitrary and will be provided in the
-- timing.res output.
simulation_name = 'Gausspulse'

-- Global parameters
rho0 = 1.
cs2  = 1./3.
cube_length = 10.

-- Include general treelm settings
require('treelm/config')

-- Some parameters for the functions given below
originX =  -1.3
originY =  0.8
originZ =  0.1
halfwidth = 0.50
amplitude = 0.01
p0 = rho0*cs2

-- Some spatial lua functions, they take three coordinates and return one value.
function ic_1Dgauss_pulse(x, y, z)
  return p0+amplitude*math.exp(-0.5/(halfwidth^2)*( x - originX )^2)
end
function ic_2Dgauss_pulse(x, y, z)
  return p0 +amplitude*math.exp(-0.5/(halfwidth^2)*(( x - originX )^2+( y - originY )^2))
end
function ic_1Dgauss_pulse2(x, y, z,t)
  return p0+amplitude*math.exp(-0.5/(halfwidth^2)*( x - originX )^2)
end

-- Define the interpolation method to use between different refinement levels
-- available are: average, eqneq, quadratic, quadraticCompact
interpolation_method = 'average' -- default: average

-- Time step settings
tmax           =  20     -- some helping variable to use below
-- Actual time definitions
-- times can be given in terms of iterations or simulation time.
time = {useIterations = true, -- Timings given in iterations? default: true
        max = tmax,           -- Maximal iteration to reach
        interval = tmax+1 }    -- Interval for checking density

io_buffer_size = 8    -- in term of MB, default value is 8

printRuntimeInfo = true   -- print runtime info after simulation

control_routine = 'multischeme'  -- another option is 'benchmark'

scaling = 'acoustic' -- or diffusive

comm_reduced = true   -- communicate reduced set of links?

init_allElems = false -- besides fluids, initialize ghost and halo elements?
                      -- only use this feature for debugging!

init_allTimeLayers = true -- initialize all time layers

-- mpi communication type, other options are:
-- isend_irecv_overlap
-- typed_isend_irecv
-- gathered_type
commpattern = 'isend_irecv'


-- Debug options
debug = { debugMode = true,  -- activate global debug modus
          debugFiles = true, -- open the debugFiles for each process dbgOut0000**.out
          dumpDependencies = true -- write the dependencies between levels to disk
          }

-- scheme model for single fluid simulation
scheme = {
  identify = {
      label = 'global',  -- scheme name 
      kind = 'lbm',      -- simulation type of this scheme ( lbm, lbm_incomp, lbm_ps, ...)
      relaxation = 'bgk', -- relaxation type (bgk, mrt, ...)
-- Scheme layout
-- This describes the stencil to use in the simulation.
      layout = 'd3q19'},

-- Physical reference values, used for LB to physical unit convertion
-- physics = { setPhysicalQuantities = true,
--             dt = dt_phy,
--             rho0 = rho0_phy,
--             viscosity = vis0_phy,
--             bulk_viscosity = bulk_vis0_phy,
--             molWeigh0 = mol_phy,
--             temp0 = t_phy, },

  -- field which defines fluid or species
  -- Single fluid simulation
--  field = {
--     label = 'fluid',
-- Define the Lattice Boltzmann fluid properties
     fluid = { omega = 1.7, rho0 = rho0 },
-- Initial condition for each field
-- Define initial conditions per variable.
-- They might be constants, predefined functions (described with a table), or
-- lua functions.
initial_condition = { 
--     density =  {
--       predefined='gausspulse', 
--       center={5.0,5.0, 5.0}, 
--       halfwidth=1., 
--       amplitude=1.20, 
--       background=1.000},
-- initial with lua function
     pressure  = ic_1Dgauss_pulse, -- see above for its definition / computation.
     velocityX = 0.0,  -- constant IC
     velocityY = 0.0,  -- constant IC
     velocityZ = 0.0 } -- constant IC
-- boundary condition for each field
--    boundary_condition = {{}}
--              }

--,add_variable = {{'spacetime',1,ic_1Dgauss_pulse2}}

-- See treelm/config.lua for explanations on tracking settings.
, tracking = {
   label = 'point',
   variable = { {'density'}, {'velocity'}
--    { name = 'spacetime', ncomponents = 1, spacetime = ic_1Dgauss_pulse2 },
--    { 'spacetime' },
--    { name = 'difference', ncomponents = 1, dep = {'density','spacetime'} },
-- possible names are:
--    pdf, density, pressure, velocity, velMag,
--    temperature, omega, molfrac, kinetic_energy
--    equilFromState, equilFromMacro, nonEquilFromState,
--    shearstress, wss, shearMag
--    spacetime, difference
--    {'velocity',sub_index={1}}
              }, -- variable table
   folder = 'tracking_',
   shape = {kind = 'canoND', object = {origin = {3.0,3.1,3.0} } },
--   shape = { kind = 'canoND', object = {origin = {0.0, 0.0, 0.0}, 
--                                        vec = {0.0, 10.0, 0.0}, 
--                                        segments = 100,
--                                        distribution = 'equal'}},
--    track boundary elements
--    shape = {kind = 'property', property = {'boundary'}},
   format = 'ascii',
   time = {min = 10, max = tmax, interval = 1},
  }   -- tracking table
    }
--}

-- scheme model for multispecies simulation
--scheme = {
--  identifier = {
--      label = 'default',
--      kind = 'LBM_multispecies', 
--      relaxation = 'BGK', 
---- scheme layout
--      layout = 'd3q19'},
--      depend = { vay_sys = {variable={'velocity'}}, usescheme='fluid'},
--  field = {
--   {
--   label = 'spc1'
--   species = { 
--        molecular_weight = 1.0,
--        diff_coeff = 1.0
--       }
--    -- Initial condition 
--     initial_condition = { 
---- initial with lua function
--              density = ic_2Dgauss_pulse
--              velocity = {0.0,0.0,0.0}}
---- boundary_condition = {{}}
--   
--   },
--   {
--   label = 'spc2'
--   species = { 
--        molecular_weight = 2.0,
--        diff_coeff = 1.0
--       }
---- Initial condition 
--     initial_condition = { 
---- initial with lua function
--              density = ic_2Dgauss_pulse
--              velocityX = 0.0,
--              velocityY = 0.0,
--              velocityZ = 0.0 }
---- boundary_condition = {{}}
--
--   }
--  }
--}
--
