! See copyright notice in the COPYRIGHT file.
?? include 'header/lbm_macros.inc'
!******************************************************************************!
!> Boundary condition treatment routines for fluid simulation
!!
!! A detailed description on the implementation details are given in
!! \ref boundary_implementation
!!
module mus_bc_fluid_experimental_module

  ! include treelm modules
  use env_module,               only: rk
  use tem_param_module,         only: div1_3, cs2inv
  use tem_spacetime_fun_module, only: tem_spacetime_for
  use tem_tools_module,         only: tem_write
  use tem_time_module,          only: tem_time_type
  use treelmesh_module,         only: treelmesh_type
  use tem_var_system_module,    only: tem_var_system_type
  use tem_subTree_type_module,  only: tem_subTree_type
  use tem_debug_module,         only: tem_debug
  use tem_logging_module,       only: tem_toStr
  use tem_ini_condition_module, only: tem_ini_condition_type

  ! include musubi modules
  use mus_bc_header_module,     only: boundary_type, glob_boundary_type
  use mus_scheme_layout_module, only: mus_scheme_layout_type 
  use mus_field_prop_module,    only: mus_field_prop_type
  use mus_pdf_module,           only: pdf_global_type
  use mus_variable_module,      only: mus_derVarPos_type
  use mus_global_module,        only: param_type
  use mus_physics_module,       only: mus_physics_type

  implicit none

  private

  public :: outlet_extrapolation
  public :: outlet_extrapolation_microscopic
  public :: outlet_bouzidi
  public :: test_bcNeighs

contains
!******************************************************************************!
  !> Outlet boundary condition with experimental higher order macroscopic 
  !! quantity extrapolation
  !! \author Manuel Hasert
  !!
  !!  with higher order macroscopic value extrapolation
  !! This is only experimental and follows no paper reference.
  !! We use an extrapolation of the macroscopic values and then do a anti bounce
  !! back with the extrapolated values
  !!
  ! subroutine outlet_extrapolation
?? copy :: bc_routineHeader( outlet_extrapolation )
    !---------------------------------------------------------------------------
    real(kind=rk) :: fEqPlus, fPlus, omega
    real(kind=rk) :: uxB(3), uxB_1(3), uxB_2(3), uxB_3(3), uxB_4(3)
    real(kind=rk) :: rhoB, rhoB_1, rhoB_2, rhoB_3, rhoB_4
    real(kind=rk) :: fTmp_1( layout%stencil(1)%QQ )
    real(kind=rk) :: fTmp_2( layout%stencil(1)%QQ )
    real(kind=rk) :: fTmp_3( layout%stencil(1)%QQ )
    real(kind=rk) :: fTmp_4( layout%stencil(1)%QQ )
    real(kind=rk) :: usq!, rho_temp(1) 
    integer :: iELem, iDir
    ! local DUMMY subTree needed to use the function pointers (not filled)
    type(tem_subTree_type) :: subTree
    integer :: bound(2)
    !---------------------------------------------------------------------------
    call tem_write('MH: Warning: I didnt adapt these BCs to the new bitmasks! Wrong for sure')

    ! transient pressure value
!    rho_temp = tem_transient_for( me%bc_states%pressure%transient, real(timing%iTime,kind=rk) )
    omega = fieldProp%fluid%omLvl( iLevel )

    do iElem = 1, globBC%nElems( iLevel )
      ! Get local and neighbor velocity
      rhoB_1 = 0.0_rk
      rhoB_2 = 0.0_rk
      rhoB_3 = 0.0_rk
      rhoB_4 = 0.0_rk
      do iDir = 1,layout%stencil(1)%QQ
        fTmp_1( iDir ) = bcBuffer( &
          & ?IDX?( varPos(iDir), globBC%elems(iLevel)%elemBuffer(iElem), nScalars ) )
        fTmp_2 = me%elems( iLevel )%neighBufferPre( 1, &
          & ?IDX?( iDir, iElem, layout%stencil(1)%QQ ) )
        if( me%order .gt. 1) &
          & fTmp_3 = me%elems( iLevel )%neighBufferPre( 2, &
          & ?IDX?( iDir, iElem, layout%stencil(1)%QQ ) )

        if( me%order .gt. 2) &
          & fTmp_4 = me%elems( iLevel )%neighBufferPre( 3, &
          & ?IDX?( iDir, iElem, layout%stencil(1)%QQ ) )
      enddo
     
      rhoB_1 = sum(fTmp_1)
      rhoB_2 = sum(fTmp_2)
      if( me%order .gt. 1)  rhoB_3 = sum(fTmp_3)
      if( me%order .gt. 2)  rhoB_4 = sum(fTmp_4)
     
      bound(1)=iElem
      bound(2)=iElem
      call varSys%derive( derVarPos%velocity )%do(                             &
        &                 input   = fTmp_1,                                    &
        &                 bound   = bound,                                     &
        &                 tree    = tree,                                      &
        &                 subTree = subTree,                                   &
        &                 requVar = varSys%Variable%val(derVarPos%velocity),   &
        &                 globSys = varSys,                                    &
        &                 stencil = layout%stencil(1),                         &
        &                 conf    = varSys%solSpec_conf,                       &
        &                 res     = uxB_1 )

      call varSys%derive( derVarPos%velocity )%do(                             &
        &                 input   = fTmp_2,                                    &
        &                 bound   = bound,                                     &
        &                 tree    = tree,                                      &
        &                 subTree = subTree,                                   &
        &                 requVar = varSys%Variable%val(derVarPos%velocity),   &
        &                 globSys = varSys,                                    &
        &                 stencil = layout%stencil(1),                         &
        &                 conf    = varSys%solSpec_conf,                       &
        &                 res     = uxB_2 )

     if( me%order .gt. 1)                                                      &
       &call varSys%derive( derVarPos%velocity )%do(                           &
          &                 input   = fTmp_3,                                  &
          &                 bound   = bound,                                   &
          &                 tree    = tree,                                    &
          &                 subTree = subTree,                                 &
          &                 requVar = varSys%Variable%val(derVarPos%velocity), &
          &                 globSys = varSys,                                  &
          &                 stencil = layout%stencil(1),                       &
          &                 conf    = varSys%solSpec_conf,                     &
          &                 res     = uxB_3 )

     if( me%order .gt. 2)                                                      &
       &call varSys%derive( derVarPos%velocity )%do(                           &
          &                 input   = fTmp_4,                                  &
          &                 bound   = bound,                                   &
          &                 tree    = tree,                                    &
          &                 subTree = subTree,                                 &
          &                 requVar = varSys%Variable%val(derVarPos%velocity), &
          &                 globSys = varSys,                                  &
          &                 stencil = layout%stencil(1),                       &
          &                 conf    = varSys%solSpec_conf,                     &
          &                 res     = uxB_4 )

      ! Extrapolate velocities on the boundary node
      ! Note here, that the position is q=0.5
      ! Other interpolation formula required for different q
      select case( me%order )
      case( 1 ) ! Linear
        uxB  = 1.5_rk*uxB_1 - 0.5_rk*uxB_2
        rhoB = 1.5_rk*rhoB_1 - 0.5_rk*rhoB_2
      case( 2 ) ! Quadratic
        uxB  = 1.875_rk*uxB_1 - 1.25_rk*uxB_2 + 0.375_rk*uxB_3
        rhoB = 1.875_rk*rhoB_1 - 1.25_rk*rhoB_2 + 0.375_rk*rhoB_3
      case( 3 ) ! Cubic
        uxB  = 2.1875_rk*uxB_1 - 2.1875_rk*uxB_2 + 1.3125_rk*uxB_3             &
          &  - 0.3125_rk*uxB_4
        rhoB = 2.1875_rk*rhoB_1 - 2.1875_rk*rhoB_2 + 1.3125_rk*rhoB_3          &
          &  - 0.3125_rk*rhoB_4
      end select

      ! Outlet pressure specified in configuration file
      ! rho = me%bc_states%pressure
      ! transient pressure  time spatial pressure
!      rhoB = rho_temp(1) * globBC%elems( iLevel )%spatial_term( 1, iElem )

      usq = uxB(1)*uxB(1) + uxB(2)*uxB(2) + uxB(3)*uxB(3) 

      do iDir = 1,layout%stencil(1)%QQ
        if( globBC%elems(iLevel)%bitmask( iDir, iElem )) then
          ! Calc the correction term and the Dirichlet (Eq) term
          fEqPlus = layout%weight( iDir ) * rhoB                               &
            &     + 4.5_rk*layout%weight( iDir )*fieldProp%fluid%rho0*((       &
            &       layout%stencil(1)%cxDir( 1, iDir )*uxB(1)                  &
            &     + layout%stencil(1)%cxDir( 2, iDir )*uxB(2)                  &
            &     + layout%stencil(1)%cxDir( 3, iDir )*uxB(3))**2              &                
            &     - div1_3*usq)                
          fPlus = 0.5_rk*(                                                     &
  state( ?SAVE?( varPos(iDir), globBC%elems(iLevel)%elem( iElem ), nScalars, neigh ), currT) &
+ state( ?SAVE?( varPos( layout%stencil(1)%cxDirInv(iDir )), globBC%elems(iLevel)%elem( iElem ), nScalars, neigh ) &
        , currT))

 ! Now assign the values
  state( ?SAVE?( varPos(iDir), globBC%elems(iLevel)%elem( iElem ), nScalars, neigh ), currT)  = &
 ! antibounceback term
 - state( ?SAVE?( varPos(iDir), globBC%elems(iLevel)%elem( iElem ), nScalars, neigh ), currT)   &
          ! Dirichlet pressure term
          & + 2._rk*fEqPlus                                                                     &
          ! Correction term
          & + (2._rk - omega )*( fPlus - fEqPlus )
        endif
      enddo
    enddo
 
  end subroutine outlet_extrapolation
!******************************************************************************!


!******************************************************************************!
  !> Outlet boundary conditions with zero-f-gradient
  !! \author Manuel Hasert
  !!
  !! Simple microscopic extrapolation outlet boundaries with zero-f-gradient.
  !! This follows equation (18) in S. Chen, D. Martinez, and R. Mei, 
  !!  "On boundary conditions in lattice Boltzmann methods," 
  !!  Physics of Fluids, vol. 8, no. 9, p. 2527, 1996.
  !! 
  !! MH: These are not working yet!!
  ! subroutine outlet_extrapolation_microscopic
?? copy :: bc_routineHeader( outlet_extrapolation_microscopic )
    !--------------------------------------------------------------------------- 
    ! defining local variables
    integer :: iELem, iDir, fluidNeighbor, fluidNeighbor2
    !--------------------------------------------------------------------------- 
    call tem_write('MH: Warning: I didnt adapt these BCs to the new bitmasks! Wrong for sure')

      select case( me%order) 
      case( 0 ) ! Zero order extrapolation
      do iElem = 1, globBC%nElems( iLevel )
        do iDir = 1,layout%stencil(1)%QQ
          if( globBC%elems(iLevel)%bitmask(iDir, iElem )) then
            ! Only treat the required directions
            ! f_i(xB) = f_i(xf)
            ! But we have already bounced back intrinsically, so we take the first f_i = f_i'
            ! and also have to trace the f_i', which was actually already pushed further with PUSH
state( ?FETCH?( varPos(iDir), globBC%elems(iLevel)%elem( iElem ),nScalars, neigh ), currT)  =       &
state( ?FETCH?( varPos( layout%stencil(1)%cxDirInv( iDir ) ), globBC%elems(iLevel)%elem( iElem ),nScalars, neigh ) &
           , currT)
          endif
        enddo
      enddo
      case( 1 ) ! Linear extrapolation
      do iElem = 1, globBC%nElems( iLevel )
        do iDir = 1,layout%stencil(1)%QQ
          if( globBC%elems(iLevel)%bitmask(iDir, iElem )) then
            ! Only treat the required directions
            ! f_i(xB) = 2*f_i(xf) - f_i(xf+1)
            ! But we have already bounced back intrinsically, so we take the first f_i = f_i'
            ! and also have to trace the f_i', which was actually already pushed further with PUSH
            fluidNeighbor =  &
?ElemIDX?(neigh( ?NGPOS?( varPos( layout%stencil(1)%cxDirInv( iDir ) ), globBC%elems(iLevel)%elem(iElem),nScalars )),nScalars)
    state( ?FETCH?( varPos(iDir), globBC%elems(iLevel)%elem( iElem ),nScalars, neigh ), currT)  =       &
    2._rk * state( &
    ?FETCH?( varPos( layout%stencil(1)%cxDirInv( iDir ) ), globBC%elems(iLevel)%elem( iElem ), nScalars, neigh ), currT)  &
 - state( ?FETCH?( varPos( layout%stencil(1)%cxDirInv( iDir ) ), fluidNeighbor, nScalars, neigh ), currT)  
          endif
        enddo
      enddo
      case( 2 ) ! Quadratic extrapolation
        do iElem = 1, globBC%nElems( iLevel )
          do iDir = 1,layout%stencil(1)%QQ
            if( globBC%elems(iLevel)%bitmask(iDir, iElem )) then
              ! Only treat the required directions
              ! f_i(xB) = 3*f_i(xf) - 3*f_i(xf+1)+ 3*f_i(xf+2)
              ! But we have already bounced back intrinsically, so we take the first f_i = f_i'
              ! and also have to trace the f_i', which was actually already pushed further with PUSH
              fluidNeighbor =  &
?ElemIDX?(neigh( ?NGPOS?( varPos( layout%stencil(1)%cxDirInv( iDir ) ), globBC%elems(iLevel)%elem(iElem),nScalars )),nScalars)
              fluidNeighbor2 = &
?ElemIDX?(neigh( ?NGPOS?( varPos( layout%stencil(1)%cxDirInv( iDir ) ), fluidNeighbor,nScalars )),nScalars)
         state( ?FETCH?( varPos(iDir), globBC%elems(iLevel)%elem( iElem ), nScalars, neigh ), currT)  =       &
         3._rk * state( &
         ?FETCH?( varPos( layout%stencil(1)%cxDirInv( iDir ) ), globBC%elems(iLevel)%elem( iElem ), nScalars, neigh )&
         , currT)  &
         - 3._rk * state( &
         ?FETCH?( varPos( layout%stencil(1)%cxDirInv( iDir ) ), fluidNeighbor, nScalars, neigh ), currT)  &
        + state( ?FETCH?( varPos( layout%stencil(1)%cxDirInv( iDir ) ), fluidNeighbor2, nScalars, neigh ),&
        currT)  
            endif
          enddo
        enddo
      end select
 
    end subroutine outlet_extrapolation_microscopic
!******************************************************************************!


!******************************************************************************!
  !> Outlet Pressure Bounce Back boundary condition 
  !! These boundary conditions are taken from the paper:  
  !! M. Bouzidi et al, “ Momentum  transfer  of  a 
  !!  Boltzmann-lattice  fluid  with  boundaries,” PHYSICS OF
  !! FLUIDS, vol. 13, no. 11, 2001.
  !!
  !>\todo: these BC are implemented like in the DC-code, they need to be tested
  !!      and checked
  ! subroutine outlet_bouzidi
?? copy :: bc_routineHeader( outlet_bouzidi )
    !---------------------------------------------------------------------------
    real(kind=rk) ux(3), usq, rho(globBC%nElems(iLevel))!, rho_temp
    integer :: iELem, iDir
    !---------------------------------------------------------------------------
    call tem_write('MH: Warning: I didnt adapt these BCs to the new bitmasks! Wrong for sure')

    ! transient pressure value
    rho = tem_spacetime_for( me      = me%bc_states%pressure%spacetime,        &
      &                      treeIds = glob%levelDesc( iLevel )%total( globBC% &
      &                                               elems(iLevel)%elem(:) ), &
      &                      tree    = tree,                                   &
      &                      time    = timing,                                 &
      &                      n       = globBC%nElems(iLevel),                  &
      &                      conf    = params%conf )

    ! If physical quantities are given, transform to lattice units by division
    ! with the conversion factor
    rho = rho/physics%fac( iLevel )%press
    ! And convert pressure to density
    rho = cs2inv*rho

    do iElem = 1, globBC%nElems( iLevel )
      ! Calculate local pressure and velocity moments
      rho = 0._rk
      do iDir = 1,layout%stencil(1)%QQ
        rho = rho + state( ?SAVE?( varPos(iDir), globBC%elems(iLevel)%elem( iElem ), nScalars, neigh ), currT)
      enddo 
      ux  = 0._rk
      do iDir = 1,layout%stencil(1)%QQ
        ux(:)  = ux(:) + layout%stencil(1)%cxDir( :, iDir)                     &
          &    * state( ?SAVE?( varPos(iDir), globBC%elems(iLevel)%elem( iElem ), nScalars, neigh ), currT)
      enddo 

      ! get the actual velocity out of the moments
      ux = ux/rho
      ! Outlet density specified in configuration file
      ! rho = me%density 
      ! transient density  time spatial density
!      rho = rho_temp * me%density%spatial_val( iLevel )%val( iElem )

      usq = ux(1)*ux(1) + ux(2)*ux(2) + ux(3)*ux(3) 

      do iDir = 1,layout%stencil(1)%QQ
        ! Write the values
        !KM:\todo @Manuel: Please check whether it is SAVE or FETCH
        if( globBC%elems(iLevel)%bitmask(iDir, iElem )) then
          state( ?SAVE?( varPos(iDir), globBC%elems(iLevel)%elem( iElem ),nScalars, neigh ), currT)  =   &
            &       layout%weight(iDir)*rho(iElem)*(2._rk+9._rk*( (            &
            &                    layout%stencil(1)%cxDir( 1, iDir )*ux(1)      &
            &             +      layout%stencil(1)%cxDir( 2, iDir )*ux(2)      &
            &             +      layout%stencil(1)%cxDir( 3, iDir )*ux(3))**2  &                
            &                 - div1_3*usq))                                   &
          & - state( ?SAVE?( varPos(iDir), globBC%elems(iLevel)%elem( iElem ),nScalars, neigh ), currT)
        endif
      enddo
    enddo
  end subroutine outlet_bouzidi
!******************************************************************************!


!******************************************************************************!
  !> Test routine for boundary neighbors
  !!
  ! subroutine test_bcNeighs
?? copy :: bc_routineHeader( test_bcNeighs )
    !---------------------------------------------------------------------------
    real(kind=rk) :: fTmp_1( layout%stencil(1)%QQ )
    ! need to compute precollision velocity of neighbor element
    real(kind=rk) :: rho_loc(1)  ! Temporary density for transient 
    real(kind=rk) :: uxB(3)      ! Velocity on boundary element
    integer :: iDir              ! Direction counter
    integer :: iElem, iNeigh     ! Element counter
    ! local DUMMY subTree needed to use the function pointers (not filled)
    type(tem_subTree_type) :: subTree
    integer :: bound(2)
    !--------------------------------------------------------------------------- 

    ! Run over all elements in list for this boundary
    call tem_debug( level = 1,                                                 &
      &             msg   = '  iElem     elemPos       Neigh      density '//  &
      &                     '    velocity' )
    do iElem = 1, globBC%nElems( iLevel )
      do iNeigh = 1, me%nNeighs
        do iDir = 1,layout%stencil(1)%QQ
          fTmp_1(iDir) = state( ?IDX?( varPos(iDir), me%elems( iLevel )%neigh(iNeigh,iElem), nScalars, neigh ), currT) 
        enddo
        bound(1)=iElem
        bound(2)=iElem
        call varSys%derive( derVarPos%density )%do(                            &
          &                input   = fTmp_1,                                   &
          &                bound   = bound,                                    &
          &                tree    = tree,                                     &
          &                subTree = subTree,                                  &
          &                requVar = varSys%Variable%val( derVarPos%velocity ),&
          &                globSys = varSys, stencil = layout%stencil(1),      &
          &                conf    = varSys%solSpec_conf,                      &
          &                res     = rho_loc )
    
        call varSys%derive( derVarPos%velocity )%do(                           &
          &                input   = fTmp_1,                                   &
          &                bound   = bound,                                    &
          &                tree    = tree,                                     &
          &                subTree = subTree,                                  &
          &                requVar = varSys%Variable%val( derVarPos%velocity ),&
          &                globSys = varSys, stencil = layout%stencil(1),      &
          &                conf    = varSys%solSpec_conf,                      &
          &                res     = uxB )

        call tem_debug( level = 1,                                             &
          &             msg   = trim( tem_toStr( iElem ))//                    &
          &                     trim( tem_toStr( glob%levelDesc( iLevel )%     &
          &                                     total( me%elems( iLevel )%     &
          &                                          neigh( iNeigh,iElem ))))//&
          &                     trim( tem_toStr( iNeigh ))//                   &
          &                     trim( tem_toStr( val=rho_loc, sep=',' ))//     &
          &                     trim( tem_toStr( val=uxB, sep=',' )))
      enddo
    enddo

  end subroutine test_bcNeighs
!******************************************************************************!


end module mus_bc_fluid_experimental_module
!******************************************************************************!
