!******************************************************************************!
!> This module contains routines which initiliaze advection relaxation and
!! flow field for lbm incompressible model.
module mus_initLBMIncomp_module
  ! include treelm modules
  use tem_aux_module, only: tem_abort
  use tem_tools_module, only: tem_write
 
  ! include musubi modules
  use mus_bgk_module, only: bgk_advRel_explicit_incomp
  use mus_d3q19_module, only: bgk_advRel_d3q19_incomp_les,                     &
    &                         bgk_advRel_d3q19_incomp,                         &
    &                         bgk_advRel_d3q19_incomp_omegaSpatial,            &
    &                         bgk_advRel_d3q19_incomp_fixedVel,                &
    &                         trt_advRel_d3q19_incomp
  use mus_d2q9_module, only: bgk_advRel_d2q9_explicit_incomp_fixedVel
  use mus_mrt_module, only: mrt_advRel_d3q19_incomp,                           &
    &                       mrt_advRel_d3q19_incomp_omSpatial,                 &
    &                       mrt_advRel_d3q19_incomp_les_omSpatial,             &
    &                       mrt_advRel_d3q19_incomp_les,                       &
    &                       mrt_advRel_d3q19_incomp_fixedVel
  use mus_scheme_type_module, only: mus_scheme_type

  implicit none

  private

  public :: mus_init_advRel_lbmIncomp

contains

!******************************************************************************!
  !> Initialize the relaxation model for lbm incompressible model 
  subroutine mus_init_advRel_LBMIncomp( scheme )
    !---------------------------------------------------------------------------
    type(mus_scheme_type), intent(inout) :: scheme !< scheme type 
    !---------------------------------------------------------------------------
    call tem_write( 'Choosing INCOMPRESSIBLE relaxation model: '             &
      &              //trim( scheme%header%relaxation ))

    select case( trim( scheme%header%relaxation ))
    case( 'bgk_explicit' )
      scheme%compute => bgk_advRel_explicit_incomp
    case( 'mrt_les' )
      scheme%compute => mrt_advRel_d3q19_incomp_les
    case( 'bgk_les' )
      scheme%compute => bgk_advRel_d3q19_incomp_les
    case( 'mrt_les_omega_spatial')
      call tem_write( 'Omega spatial ')
      scheme%compute => mrt_advRel_d3q19_incomp_les_omSpatial
    case( 'bgk_omega_spatial')
      call tem_write( 'Omega spatial ')
      select case( trim(scheme%header%layout) )
      case( 'd3q19' )
        scheme%compute => bgk_advRel_d3q19_incomp_omegaSpatial
      case default
        call tem_write( 'Omega spatial is currently only available for d2q9 ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select
      scheme%header%relaxation = 'bgk'

    case( 'bgk_fixedVel' )
      select case( trim(scheme%header%layout) )
      case( 'd2q9' )
        scheme%compute => bgk_advRel_d2q9_explicit_incomp_fixedVel
      case( 'd3q19' )
        scheme%compute => bgk_advRel_d3q19_incomp_fixedVel
      case default
        call tem_write( 'The selected stencil is not supported: ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select 
    case( 'bgk' )
      select case( trim(scheme%header%layout) )
      case( 'd3q19' )
        scheme%compute => bgk_advRel_d3q19_incomp
      case( 'new_stencil' )
        scheme%compute => bgk_advRel_explicit_incomp
      case( 'd2q9' )
        scheme%compute => bgk_advRel_explicit_incomp
      case default
        call tem_write( 'The selected stencil is not supported: ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select
    case( 'mrt' )
      select case( trim(scheme%header%layout) )
      case( 'd3q19' )
        scheme%compute => mrt_advRel_d3q19_incomp
      case default
        call tem_write( 'The selected stencil is not supported: ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select
    case( 'mrt_omega_spatial')
      call tem_write( 'Omega spatial ')
      scheme%compute => mrt_advRel_d3q19_incomp_omSpatial
    case( 'mrt_fixedVel' )
      select case( trim(scheme%header%layout) )
      case( 'd3q19' )
        scheme%compute => mrt_advRel_d3q19_incomp_fixedVel
      case default
        call tem_write( 'The selected stencil is not supported: ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select
    case( 'trt' )
      select case( trim(scheme%header%layout) )
      case( 'd3q19' )
        scheme%compute => trt_advRel_d3q19_incomp
      case default
        call tem_write( 'The selected stencil is not supported: ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select
    case default
      call tem_write( 'The selected relaxation model is not supported: '     &
        &              //trim( scheme%header%relaxation ))
      call tem_abort()
    end select

  end subroutine mus_init_advRel_LBMIncomp

end module mus_initLBMIncomp_module
