!******************************************************************************!
!> This module contains routines which initiliaze advection relaxation and
!! flow field for lbm model.
module mus_initLBM_module
  use tem_param_module, only: cs4inv, t2cs2inv, t2cs4inv, cs2inv, cs2
  use tem_aux_module, only: tem_abort
  use tem_tools_module, only: tem_write
  use tem_geometry_module, only: tem_baryOfId, tem_idOfCoord
  use tem_spatial_module, only: tem_spatial_for
  
  use mus_bgk_module, only: bgk_advRel_d3q7, bgk_advRel_explicit,              &
    &                       bgk_advRel_forcing
  use mus_d3q19_module, only: bgk_advRel_d3q19, bgk_advRel_d3q19_les,          &
    &                         bgk_advRel_d3q19_omegaSpatial,                   &
    &                         bgk_advRel_d3q19_fixedVel,                       &
    &                         trt_advRel_d3q19_fixedVel,                       &
    &                         trt_advRel_d3q19_les,                            &
    &                         trt_advRel_d3q19_les_omegaSpatial,               &
    &                         trt_advRel_d3q19
  use mus_d2q9_module, only: mrt_advRel_d2q9,                                  &
    &                         mrt_advRel_d2q9_explicitEq,                      &
    &                         bgk_advRel_d2q9_explicit_fixedVel,               &
    &                         bgk_advRel_d2q9_explicitOmegaSpatial
  use mus_mrt_module, only: mrt_advRel_d3q19,                                  &
    &                       mrt_advRel_d3q19_omSpatial,                        &
    &                       mrt_advRel_d3q19_fixedVel,                         &
    &                       mrt_advRel_d3q19_les,                              &
    &                       mrt_advRel_d3q19_les_omSpatial
  use mus_scheme_type_module, only: mus_scheme_type

  implicit none

  private

  public :: mus_init_advRel_LBM
!  public :: mus_init_LBM_withFlowParams

contains

!******************************************************************************!
  !> Initialize the relaxation model for lbm scheme kind
  subroutine mus_init_advRel_LBM( scheme )
    !---------------------------------------------------------------------------
    type(mus_scheme_type), intent(inout) :: scheme !< scheme type 
    !---------------------------------------------------------------------------
    call tem_write( 'Choosing relaxation model: '             &
      &              //trim( scheme%header%relaxation ))
    select case( trim( scheme%header%relaxation ))
    case( 'mrt_les' )
      scheme%compute => mrt_advRel_d3q19_les
    case( 'bgk_les' )
      scheme%compute => bgk_advRel_d3q19_les
    case( 'bgk_forcing' )
      scheme%compute => bgk_advRel_forcing
    case( 'mrt_les_omega_spatial')
      call tem_write( 'Omega spatial ')
      scheme%compute => mrt_advRel_d3q19_les_omSpatial
    case( 'trt_les_omega_spatial')
      call tem_write( 'Omega spatial ')
      scheme%compute => trt_advRel_d3q19_les_omegaSpatial
    case( 'bgk_omega_spatial')
      call tem_write( 'Omega spatial ')
      select case( trim(scheme%header%layout) )
      case( 'd2q9' )
      scheme%compute => bgk_advRel_d2q9_explicitOmegaSpatial
      case( 'd3q19' )
        scheme%compute => bgk_advRel_d3q19_omegaSpatial
      case default
        call tem_write( 'Omega spatial is currently only available for d2q9 ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select
      scheme%header%relaxation = 'bgk'
    case( 'bgk' )
      select case( trim(scheme%header%layout) )
      case( 'd3q19' )
        scheme%compute => bgk_advRel_d3q19
      case( 'd3q7' )
        scheme%compute => bgk_advRel_d3q7
      case( 'new_stencil' )
        scheme%compute => bgk_advRel_explicit
      case( 'd2q9' )
        scheme%compute => bgk_advRel_explicit
      case( 'd1q3' )
        scheme%compute => bgk_advRel_explicit
      case default
        call tem_write( 'The selected stencil is not supported at this moment: ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select
    case( 'bgk_explicit' )
        scheme%compute => bgk_advRel_explicit
    case( 'bgk_fixedVel' )
      select case( trim(scheme%header%layout) )
      case( 'd3q19' )
        scheme%compute => bgk_advRel_d3q19_fixedVel
      case( 'd2q9' )
        scheme%compute => bgk_advRel_d2q9_explicit_fixedVel
      case default
        call tem_write( 'The selected stencil is not supported: ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select
    case( 'trt_fixedVel' )
      select case( trim(scheme%header%layout) )
      case( 'd3q19' )
        scheme%compute => trt_advRel_d3q19_fixedVel
      case default
        call tem_write( 'The selected stencil is not supported at this moment: ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select
    case( 'trt' )
      select case( trim(scheme%header%layout) )
      case( 'd3q19' )
        scheme%compute => trt_advRel_d3q19
      case default
        call tem_write( 'The selected stencil is not supported at this moment: ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select
    case( 'trt_les' )
      select case( trim(scheme%header%layout) )
      case( 'd3q19' )
        scheme%compute => trt_advRel_d3q19_les
      case default
        call tem_write( 'The selected stencil is not supported at this moment: ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select
    case( 'mrt_fixedVel' )
      select case( trim(scheme%header%layout) )
      case( 'd3q19' )
        scheme%compute => mrt_advRel_d3q19_fixedVel
      case default
        call tem_write( 'The selected stencil is not supported: ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select
    case( 'mrt_omega_spatial' )
      scheme%compute => mrt_advRel_d3q19_omSpatial
    case( 'mrt','mrt_bgk' )
      select case( trim(scheme%header%layout) )
      case( 'd3q19' )
        scheme%compute => mrt_advRel_d3q19
      !case( 'new_stencil' )
      !  scheme%compute => bgk_advRel_mrt_explicit
      case( 'd2q9' )
        scheme%compute => mrt_advRel_d2q9
        !scheme%compute => mrt_advRel_d2q9_explicitEq
      case default
        call tem_write( 'The selected stencil is not supported: ' &
          &             //trim(scheme%header%layout) )
        call tem_abort()
      end select
    case( 'mrt_explicit' )
      case( 'd2q9' )
        call tem_write( 'Choosing explicit MRT: ')
        scheme%compute => mrt_advRel_d2q9_explicitEq
    case default
      call tem_write( 'The selected relaxation model is not supported: '     &
        &             //trim( scheme%header%relaxation ))
      call tem_abort()
    end select

  end subroutine mus_init_advRel_LBM

end module mus_initLBM_module
