! See copyright notice in the COPYRIGHT file.
!******************************************************************************!
!> This module contains mus_field_prop_type and modules related
!! to fiels properties. 
!!
!! \author Kannan Masilamani, Simon Zimny
!!
module mus_field_prop_module

  ! include treelm modules
  use treelmesh_module, only: treelmesh_type
  use tem_tools_module, only: tem_write
  use tem_aux_module,   only: tem_abort

  ! include aotus modules
  use flu_binding,     only: flu_State
  use aot_out_module,  only: aot_out_type
  use aot_path_module, only: aot_path_type, assignment(=)

  ! include musubi modules
  use mus_scheme_header_module, only: mus_scheme_header_type
  use mus_fluid_module,         only: fluid_type, mus_load_fluid, mus_fluid_out
  use mus_pdf_module,           only: pdf_globInfo_type
  use mus_physics_module,       only: mus_physics_type

  implicit none

  private

  public :: mus_field_prop_type
  public :: mus_load_field_prop
  public :: mus_field_prop_out

  !> This type contains parameter needed for field
  type mus_field_prop_type
    !> contains fluid information
    type( fluid_type ) :: fluid
  end type mus_field_prop_type

contains
 
!******************************************************************************!
  !> load fluid properties like fluid from
  !! lua file based on the scheme kind
  subroutine mus_load_field_prop( me, conf, parent, pdfGlobal, schemeHeader, &
    &                             nFields, physics, path )
    !--------------------------------------------------------------------------
    !> field property type
    type( mus_field_prop_type ), intent(out) :: me
    !> number of fields defined in lua file
    integer, intent(in) :: nFields 
    !> flu state
    type( flu_State ), intent(inout) :: conf 
    !> parent lua handle
    integer, intent(in), optional :: parent 
    type( pdf_globInfo_type) :: pdfGlobal !< global pdf info
    !> identifier of the scheme
    type( mus_scheme_header_type ), intent(in) :: schemeHeader
    !> aotus lua path
    type( aot_path_type ),intent(in) :: path 
    !> physics type to convert physics to lattice unit or vice versa
    type( mus_physics_type ), intent(in) :: physics
    !--------------------------------------------------------------------------
    call tem_write('Loading the field properties ..')
    !load fluid info
    select case( trim(schemeHeader%kind) )
    case('lbm', 'lbm_incomp')
      call mus_load_fluid( me = me%fluid,                                      &
        &                  conf = conf,                                        &
        &                  parent = parent,                                    &
        &                  pdfGlobal = pdfGlobal,                              &
        &                  path = path,                                        & 
        &                  physics = physics )
    case default
      call tem_write( 'The selected scheme kind is unknown '//                 &
        &             trim(schemeHeader%kind) )
      call tem_abort()
    end select                

  end subroutine mus_load_field_prop
!******************************************************************************!


!******************************************************************************!
  !> write field prop into a lua file
  !!
  subroutine mus_field_prop_out( me, conf, schemeHeader )
    !---------------------------------------------------------------------------
    !> single field type
    type( mus_field_prop_type ), intent(in) :: me
    !> identifier of the scheme
    type( mus_scheme_header_type ), intent(in) :: schemeHeader
    !> aotus out type
    type(aot_out_type), intent(inout) :: conf
    !---------------------------------------------------------------------------

    select case( trim(schemeHeader%kind) )
    case('lbm', 'lbm_incomp')
      call mus_fluid_out( me = me%fluid, conf= conf )
    end select  
 
  end subroutine mus_field_prop_out
!******************************************************************************!


end module mus_field_prop_module
!******************************************************************************!
