! See copyright notice in the COPYRIGHT file.
!******************************************************************************!
!> Here, the most important parameters are collected 
!! Most of them are physics and model related.
module mus_param_module

  ! include treelm modules
  use env_module, only: rk

?? text :: d3q39_parameters
  !=============================================================================
  ! D3Q39 flow model
  ! Paper Tsutahara 2008, 'New model and scheme for compressible fluids of the 
  !  FD LBM and direct simulations of aerodynamic sound'
  !=============================================================================
  !> Definition of the discrete velocity set

  integer, parameter :: QQ  = 39   !> number of pdf directions
  integer, parameter :: QQN = 38   !> number of neighbors
  ! Temporary declaration
  ! Not used
  integer,parameter :: q_BS     = 0     !<                  z-,y-
  integer,parameter :: q_TS     = 0     !<                  z+,y-
  integer,parameter :: q_BN     = 0     !<                  z-,y+
  integer,parameter :: q_TN     = 0     !<                  z+,y+
  integer,parameter :: q_BW     = 0     !<                  x-,z-
  integer,parameter :: q_BE     = 0     !<                  x+,z-
  integer,parameter :: q_TW     = 0     !<                  x-,z+
  integer,parameter :: q_TE     = 0     !<                  x+,z+
  integer,parameter :: q_SW     = 0     !<                  y-,x-
  integer,parameter :: q_NW     = 0     !<                  y+,x-
  integer,parameter :: q_SE     = 0     !<                  y-,x+
  integer,parameter :: q_NE     = 0     !<                  y+,x+

  integer,parameter :: q__E     = 1     !< 
  integer,parameter :: q__W     = 2     !< 
  integer,parameter :: q__N     = 3     !< 
  integer,parameter :: q__S     = 4     !< 
  integer,parameter :: q__T     = 5     !< 
  integer,parameter :: q__B     = 6     !< 
  integer,parameter :: q_EE     = 7     !< 
  integer,parameter :: q_WW     = 8     !< 
  integer,parameter :: q_NN     = 9     !< 
  integer,parameter :: q_SS     = 10    !< 
  integer,parameter :: q_TT     = 11    !< 
  integer,parameter :: q_BB     = 12    !< 
  integer,parameter :: qEEE     = 13    !< 
  integer,parameter :: qWWW     = 14    !< 
  integer,parameter :: qNNN     = 15    !< 
  integer,parameter :: qSSS     = 16    !< 
  integer,parameter :: qTTT     = 17    !< 
  integer,parameter :: qBBB     = 18    !< 
  integer,parameter :: qNNEE    = 19    !< 
  integer,parameter :: qNNWW    = 20    !< 
  integer,parameter :: qSSWW    = 21    !< 
  integer,parameter :: qSSEE    = 22    !< 
  integer,parameter :: qTTNN    = 23    !< 
  integer,parameter :: qTTSS    = 24    !< 
  integer,parameter :: qBBSS    = 25    !< 
  integer,parameter :: qBBNN    = 26    !< 
  integer,parameter :: qTTEE    = 27    !< 
  integer,parameter :: qTTWW    = 28    !< 
  integer,parameter :: qBBWW    = 29    !< 
  integer,parameter :: qBBEE    = 30    !< 
  integer,parameter :: qTNE     = 31    !< 
  integer,parameter :: qTNW     = 32    !< 
  integer,parameter :: qTSW     = 33    !< 
  integer,parameter :: qTSE     = 34    !< 
  integer,parameter :: qBNE     = 35    !< 
  integer,parameter :: qBNW     = 36    !< 
  integer,parameter :: qBSW     = 37    !< 
  integer,parameter :: qBSE     = 38    !< 
  integer,parameter :: q__0     = 39    !< rest density is last

  !> cartesian directions of discrete velocities 
  integer, dimension(QQ,3),parameter  :: cxDir =                               & 
                            !                 !                 !                       !                                           
  reshape((/ 1,-1, 0, 0, 0, 0, 2,-2, 0, 0, 0, 0, 3,-3, 0, 0, 0, 0, 2,-2,-2, 2, 0, 0, 0, 0, 2,-2,-2, 2, 1,-1,-1, 1, 1,-1,-1, 1, 0, &
             0, 0, 1,-1, 0, 0, 0, 0, 2,-2, 0, 0, 0, 0, 3,-3, 0, 0, 2, 2,-2,-2, 2,-2,-2, 2, 0, 0, 0, 0, 1, 1,-1,-1, 1, 1,-1,-1, 0, &
             0, 0, 0, 0, 1,-1, 0, 0, 0, 0, 2,-2, 0, 0, 0, 0, 3,-3, 0, 0, 0, 0, 2, 2,-2,-2, 2, 2,-2,-2, 1, 1, 1, 1,-1,-1,-1,-1, 0/),&
 (/QQ,3/))

  !> prevailing directions directions(subset of above ), already normalized
  real(kind=rk), dimension(6,3),parameter  :: prevailDir =                     & 
  reshape((/ 1._rk, 0._rk, 0._rk,divSqrt2_2,0._rk     ,divSqrt2_2,             &
             0._rk, 1._rk, 0._rk,divSqrt2_2,divSqrt2_2,0._rk     ,             &
             0._rk, 0._rk, 1._rk,0._rk     ,divSqrt2_2,divSqrt2_2 /),(/6,3/))


  integer, dimension(QQ),parameter  :: cInvDir =   (/                          & 
    &     q__W,   q__E,   q__S,   q__N,   q__B,   q__T,                        &   
    &     q_WW,   q_EE,   q_SS,   q_NN,   q_BB,   q_TT,                        &   
    &     qWWW,   qEEE,   qSSS,   qNNN,   qBBB,   qTTT,                        &   
    &    qSSWW,  qSSEE,  qNNEE,  qNNWW,                                        &   
    &    qBBSS,  qBBNN,  qTTNN,  qTTSS,                                        &   
    &    qBBWW,  qBBEE,  qTTEE,  qTTWW,                                        &   
    &     qBSW,   qBSE,   qBNE,   qBNW,                                        &   
    &     qTSW,   qTSE,   qTNE,   qTNW,                                        &   
    &     q__0    /) 

  ! weights of the corresponding directions
  !> rest weight
  real(kind=rk), parameter :: zeroWeight = 1._rk/ 3._rk
  !> first neighbor direction weight
  real(kind=rk), parameter :: frstWeight = 1._rk/18._rk
  !> second neighbor direction weight
  real(kind=rk), parameter :: scndWeight = 1._rk/36._rk
  !> third neighbor direction weight
  real(kind=rk), parameter :: thrdWeight = 1._rk/36._rk

  real(kind=rk), dimension(QQ)    :: tWeight   

?? end text d3q39_parameters


end module mus_param_module
!******************************************************************************!
