! See copyright notice in the COPYRIGHT file.
!******************************************************************************!
!> Initialize LB related unit stuff
!!
module mus_physicsLB_module

  ! include treelm modules
  use env_module,        only: rk, globalMaxLevels
  use treelmesh_module,  only: treelmesh_type
  use tem_aux_module,    only: tem_abort
  use tem_tools_module,  only: tem_write

  ! include musubi modules
  use mus_global_module, only: param_type
  use mus_pdf_module,    only: pdf_globInfo_type

  implicit none
  private
  public :: mus_setLBdxdt
   
  contains

!******************************************************************************!
  !> Initialize flow field 
  !!
  subroutine mus_setLBdxdt( pdfGlobal, params )
    !---------------------------------------------------------------------------
    type(param_type), intent(inout) :: params  !< Global parameters
    type(pdf_globInfo_type), intent(inout) :: pdfGlobal !< global info type
    !---------------------------------------------------------------------------
    integer :: iLevel
    real(kind=rk) :: dtLB(globalMaxLevels)
    !---------------------------------------------------------------------------
 
    ! Set LB dx
    do iLevel = pdfGlobal%minLevel, pdfGlobal%maxLevel
      pdfGlobal%dxLB( iLevel ) = 2._rk**( pdfGlobal%minLevel - iLevel )
    end do
    ! Set acoustic or diffusive scaling parameters
    select case( trim( params%scaling ))
    case('acoustic')
      ! MH: Already set in mus_config
      params%scaleFactor = 2
      ! Time step for each level: ratio 1:2 from level to level
      do iLevel = pdfGlobal%minLevel, pdfGlobal%maxLevel
        dtLB( iLevel ) = 2._rk**( pdfGlobal%minLevel - iLevel ) 
      end do
    case('diffusive')
      ! MH: Already set in mus_config
      params%scaleFactor = 4
      ! Time step for each level: ratio 1:4 from level to level
      do iLevel = pdfGlobal%minLevel, pdfGlobal%maxLevel
        dtLB( iLevel ) = 2._rk**(2*( pdfGlobal%minLevel - iLevel ))
      end do
    case default 
      call tem_write(' Error: wrong scaling chosen!   ')
      call tem_write(' Choose either acoustic or diffusive   ')
      call tem_abort()
    end select
    do iLevel = pdfGlobal%minLevel, pdfGlobal%maxLevel
      pdfGlobal%dtLB( iLevel ) = real(params%scaleFactor, kind=rk)**( &
      & pdfGlobal%minLevel - iLevel ) 
      if( pdfGlobal%dtLB( iLevel ) .ne. dtLB( iLevel )) then
        call tem_write('error in dtLB computation. differs from old one.')
        call tem_abort()
      end if
    end do

  end subroutine mus_setLBdxdt
!******************************************************************************!


end module mus_physicsLB_module
!******************************************************************************!
