-- Musubi configuration file. 
-- This is a LUA script.

-- Simulation name
simulation_name = 'channel'
mesh = 'mesh/'  -- Mesh information
-- Time step settings
tmax           =  100  -- total iteration number
check_interval =  tmax+1   -- iteration check interval

ebug = { debugMode = true, debugFiles = true,
          debugMesh = './debug/mesh_', debugStates = { 
  write = {
    folder    = './debug/',    -- the folder the restart files are written to
    interval  = 1,           -- dump restart file interval
    tmin      = 1,           -- first timestep to output
    tmax      = tmax+1       -- last timestep to output
    }
 }} 

-- Initial condition 
initial_condition = { density = 1.0, 
                      velocityX = 0.0,
                      velocityY = 0.0,
                      velocityZ = 0.0 }

-- Boundary conditions
boundary_condition = {  
{ label = 'Inlet', 
  kind = 'inlet_ubb', 
  velocityX = 0.06, velocityY = 0.0, velocityZ = 0.0 }
,{ label = 'bottom', kind = 'wall'} 
,{ label = 'top', kind = 'wall'} 
,{ label = 'Front', kind = 'wall'} 
,{ label = 'Back', kind = 'wall'} 
,{ label = 'Outlet', 
  kind = 'outlet_pab',
  density = 1.0 }
}

fluid = { relaxation_scheme = 'BGK', omega = 1.8, rho0 = 1.0 }

-- Output settings
--output = { active = false , -- VTK output activated?
--  folder = './output/',     -- Output location
--  vtk = true,             -- VTK output activated?
--  ascii = true,           -- VTK output activated?
--  interval = tmax/100,        -- output interval 
--  tmin = 0,               -- first iteration to output
--  tmax = -1 }             -- last iteration to output

-- Tracking              
tracking = {
{ label = 'probe_press', 
  variables = {'density'}, 
  type='point', point ={0.,0.,0.},
  interval = 2, tmin = 0, tmax = -1
--  ,format='harvester', folder='tracking/'
  }
,
{ label = 'plane', 
  variables = {'state'}, -- state (=pdfs), pressure, density etc
  type='plane', plane ={ 
     origin = {-75.,-15.,0.}, 
     direction1 = {150.,0,0.}, 
     direction2 = {  0.,30,0.}, 
     segments1=100, 
     segments2=100, 
     distribution='equal'}, 
  format='harvester', folder='tracking/', 
  interval = 1, tmin = 0, tmax = -1 } 
-- ,
-- { label = 'line', 
-- variables = {'state'}, -- state (=pdfs), pressure, density etc
-- type='line', line ={ 
--    origin = {-75.,-0.,0.}, 
--    direction = {150.,0,0.}, 
--    segments=300, 
--    distribution='equal'}, 
-- format='harvester', folder='tracking/', 
-- interval = 1, tmin = 0, tmax = 800 } 
  }

-- Restart settings
restart_deactivated = {
  -- If this table is available it will write a restart file
  -- with the specified options.
  write = {
    folder    = 'restart/', -- the folder the restart files are written to
    interval  = 1,                                     -- dump restart file interval
    tmin      = tmax+1,                                   -- first timestep to output
    tmax      = tmax+1                                    -- last timestep to output
    }
  }
