----------------------- PLEASE READ THIS ---------------------------!!!

-- This input file is set up to run for regression check
-- Please make sure you DO NOT MODIFY AND PUSH it to the repository
                                                                          
--------------------------------------------------------------------!!!


-- Musubi configuration file. 

require "common"
inlet = 'ubb'
outlet = 'expol'--os.getenv("outlet")
prefix = 'Re'..Re..'/inlet_'..inlet..'_outlet_'..outlet..'/'
tracking_fol = './'--tracking/'--prefix..subprefix..'tracking/'
output_fol = 'output/'--prefix..subprefix..'output/'
restart_fol = 'restart/'--prefix..subprefix..'restart/'

-- This is a LUA script.
dx = getdxFromLevel( {len_bnd=length_bnd, level=level})
dt = getdtFromVel( {dx = dx, u_p = u_in_phy, u_l = u_in_L } )
omega = getOmegaFromdt( {dx=dx, dt=dt, nu_p = nu_phy } )

-- Simulation name
simulation_name = 'channel2D'
mesh = 'mesh/' -- Mesh information
printRuntimeInfo = false

-- Time step settigs
tmax_p = 1 --s
tmax     =  tmax_p/dt --nSteps     -- total iteration number
interval = tmax/10
time = {useIterations = true,
        min = 0, max = tmax, interval = interval}

-- restart 
estart = {
      ead = restart_fol..'channel2D_lastHeader.lua',
      write = restart_fol,
      time = { min = -1, max = tmax, interval = tmax}
 }


ebug = { debugMode = true, debugFiles = true,
          debugMesh = './debug/mesh_', debugStates = { 
  write = {
    folder    = './debug/',    -- the folder the restart files are written to
    interval  = 1,           -- dump restart file interval
    tmin      = 1,           -- first timestep to output
    tmax      = tmax+1       -- last timestep to output
    }
 }} 


-- needed to dump variable in physical unit
physics = { dt = dt, rho0 = rho0_p, setPhysicalQuantities = false }            

fluid = { omega = omega, 
          rho0 = rho0_l }

interpolation_method = 'linear'                     

-- Initial condition 
initial_condition = { pressure = p0_l, 
                      velocityX = 0.0,
                      velocityY = 0.0,
                      velocityZ = 0.0 }

identify = {label='2D',layout='d3q19',kind='lbm'}
-- Boundary conditions
boundary_condition = {  
{ label = 'west', 
  kind = 'inlet_'..inlet, 
--  velocityX = u_in,
  velocityX = { kind = 'combined',
                transient= {predefined='smooth', min_factor = 0.0, 
                            max_factor=1.0, from_time=0, to_time=100.0}, 
                spatial = {predefined='parabol', 
                          shape = { kind = 'canoND', object = {origin  = {0.0,0.0,zpos},
                                                               vec = {0.0,height,0.0} } 
                                  },
		          amplitude = u_in_L		  
                         }         
--                spatial = u_inflow
                                    
	      },
   velocityY = 0.0, velocityZ = 0.0},
{ label = 'east',
--   kind = 'outlet_zero_prsgrd', 
--   kind = 'outlet_eq',
--   kind = 'outlet_pab',
--   kind = 'outlet_dnt',
   kind = 'outlet_'..outlet,
   pressure = p0_l }, 
{ label = 'north', 
   kind = 'wall' },
{ label = 'south', 
   kind = 'wall' },
{ label = 'sphere', 
   kind = 'wall' }
 }


-- Output settings
utput = { active = true, -- VTK output activated?
  folder = output_fol,     -- Output location
  vtk = true,             -- VTK output activated?
  time = {min = 0, max = tmax, interval = interval}
}

-- Tracking              
tracking = {
{
  label = 'probe_pressure', 
  folder = tracking_fol,
  variable = {'pressure_phy'}, 
  shape = {
          {kind = 'canoND', object = {origin ={0.15-dx,0.2,zpos} }},
          {kind = 'canoND', object = {origin ={0.25+dx,0.2,zpos} }}
          },
  time = {min = 0, max = tmax, interval = 10},
  format = 'ascii'      
},
--{
--  label = 'probe_velocity', 
--  folder = tracking_fol,
--  variable = {'velocity_phy'}, 
--  shape = {{kind = 'canoND', object = {origin = {dx+dx/2,0.2,zpos}} },
--           {kind = 'canoND', object = {origin = {length*0.5,0.2,zpos}}}},
--  time = {min = 0, max = tmax, interval = 10},
--  format = 'ascii'      
--},
--{
--  label = 'line_velocity_in', 
--  folder = tracking_fol,
--  variable = {'velocity_phy'}, 
--  shape = {kind = 'canoND', object = {origin = {dx_half,0.0,zpos},
--                                     vec = {0.0,height,0.0},
--                                     segments = nHeight+2}},
--  time = {min = tmax, max = tmax, interval = tmax},
--  format = 'asciiSpatial'      
--},
--{
--  label = 'line_velocity_inmean', 
--  folder = tracking_fol,
--  variable = {'velocity_phy'}, 
--  shape = {kind = 'canoND', object = {origin = {0.1/2.0,0.0,zpos},
--                                     vec = {0.0,height,0.0},
--                                     segments = nHeight+2}},
--  reduction = 'average',                                   
--  time = {min = 0, max = tmax, interval = tmax},
--  format = 'asciiSpatial'      
--},
--
--{
--  label = 'line_velocity_middle', 
--  folder = tracking_fol,
--  variable = {'velocity_phy'}, 
--  shape = {kind = 'canoND', object = {origin = {length*0.5,0.0,zpos},
--                                     vec = {0.0,height,0.0},
--                                     segments = nHeight+2 }},
--  time = {min = tmax, max = tmax, interval = tmax},
--  format = 'asciiSpatial'      
--},
--
--{
--  label = 'line', 
--  folder = tracking_fol,
--  variable = {{'pressure_phy'},{'velocity_phy'}}, 
--  shape = {kind = 'canoND', object = {origin = {0.0,0.2,zpos},
--                                     vec = { length, 0.0,0.0},
--                                     segments = nLength+2}
--                                         },
--  time = {min = tmax, max = tmax, interval = tmax},
--  format = 'asciiSpatial'      
--}

}

