-- Musubi configuration file. 
-- This is a LUA script.
originX = 00.0
originY = 00.0
originZ = 00.0
halfwidth = 3.0
amplitude = 0.1
background = 1.0

function ic_1Dgauss_pulse(x, y, z)
  return background+amplitude*math.exp(-math.log(2.)/(halfwidth^2)*( x - originX )^2)
end
function ic_2Dgauss_pulse(x, y, z)
  return background+amplitude*math.exp(-math.log(2.)/(halfwidth^2)*(( x - originX )^2+( y - originY )^2))
end
-- Initial condition 
initial_condition = { --density = {predefined='gausspulse', center={0.0,0.0,0.0}, halfwidth=0.3, amplitude=0.20, background=1.000},
                      density = ic_2Dgauss_pulse,
                      velocityX = 0.0,
                      velocityY = 0.0,
                      velocityZ = 0.0 }
-- Simulation name
simulation_name = 'Gaussian_pulse_validation'
mesh = 'mesh/'-- Mesh information

fluid = { relaxation_scheme = 'BGK', omega = 1.98, rho0 = 1.0 }

-- Boundary conditions
boundary_condition = {  
{ label = 'periodic',
   kind = 'periodic' }
}


-- Local refinement settings
interpolation_method = 'linear'
                     
-- Time step settigs
tmax           =  120    -- total iteration number
check_interval =  tmax/10   -- iteration check interval

-- Output settings
output = { active = false, -- VTK output activated?
  folder = 'output/',     -- Output location
  vtk = true,             -- VTK output activated?
  interval = 4, -- tmax/40,    -- output interval 
  dumpGhosts = true,
  tmin = 0,               -- first iteration to output
  tmax = -1 }             -- last iteration to output

-- Tracking              
tracking = {
{ label = 'probe_press', 
  variables = {'density'}, 
  type='point', point ={0.,0.,0.},
  interval = 2, tmin = 0, tmax = -1,
  format='ascii', folder='tracking/'
  }
,
{ label = 'line', 
  variables = {'density'}, -- state (=pdfs), pressure, density etc
  type='line', line ={ 
     origin = {0.,0.,0.}, 
     direction = {60.,0.,0.}, 
     segments=200, 
     distribution='equal'},
     format='asciiSpatial', folder='tracking/', 
     interval = tmax/tmax, tmin = tmax, tmax = -1 } 
-- ,
-- { label = 'line', 
-- variables = {'state'}, -- state (=pdfs), pressure, density etc
-- type='line', line ={ 
--    origin = {-75.,-0.,0.}, 
--    direction = {150.,0,0.}, 
--    segments=300, 
--    distribution='equal'}, 
-- format='harvester', folder='tracking/', 
-- interval = 1, tmin = 0, tmax = 800 } 
  }

