--This file contains common parameters

-- Setting mesh refinement level
-- get the value from BASH environment variable level
-- OR get level from user input
level = os.getenv('level')
if level == nil then
  print("Please input the mesh refinement level: (6, 7, 8 or 9)")
  level = io.read("*number")
  print("Mesh refinement level is: ",level)
end
Reynolds = os.getenv('Re')
if Reynolds == nil then
  print("Please input the Reynolds number:")
  Reynolds = io.read("*number")
  print("Reynolds number is: ",Reynolds)
end

diffusive = true  -- diffusive or acoustic scaling?
usePeriodic = true -- true = 2D channel, false = 3D channel
useObstacle = false -- not used yet
multi_level = 0   -- 0 = single-level, 1 = multi-level
interpolation_method = 'quadratic'
printRuntimeInfo = false
scaling = 'diffusive'
identify_kind = 'lbm_incomp'
identify_kind = 'lbm'

mesh = 'mesh/' -- Mesh files location

length = 8.0 * 0.01    -- physical length of channel
height = length / 8

-- Do not change these values.
-- They are taken as a base to calculate the position of the periodic plane
dx = length / 2.^level
ly = height / dx       -- number of elements for channel height
lx = length / dx - 2   -- number of elements for channel height
R = ly / 2    -- channel radius
R = height / 2

-- For Grid convergence,
-- keep Re = U_aver * Ly / viscosity to be constant
if diffusive then
  -- For DIFFUSIVE scaling, keep omega constant, adjust velMax accordingly
  omega = 1.9
  velMaxLB = ( 2 * Reynolds / omega - Reynolds ) / 4 / ly
  tmaxLB  =  5000 * ( 4.^(level - 6 ))
else
  -- For ACOUSTIC scaling, keep vel constant, adjust omega accordingly
  velMaxLB = 0.005
  omega = 2 * Reynolds / ( 4.0 * velMax * ly + Reynolds )
  tmaxLB  =  9000 * ( 2.^(level - 6 ))
end

-- Physical parameters
densityPhy = 1050
viscPhy = 0.0035
nuPhy = viscPhy / densityPhy

-- reference LB quantities
rho0 = 1.
cs2  = 1./3.
p0LB   = rho0*cs2
nuLB = ( 1.0 / omega - 0.5 ) / 3.0   -- viscosity
dt = nuLB * dx * dx / nuPhy
tmax = tmaxLB

velMaxPhy = velMaxLB * dx / dt
p0Phy = p0LB * densityPhy * dx * dx / dt / dt
tmaxPhy = tmaxLB * dt

--D_slope = 8.0 * nuLB * velMax / ly / ly * 3 -- density drop slope
--dt = getdtFromOmega( { dx = dx, nu_p = vis_phy, omega = omega} )
--dt = Reynolds * dx * dx * vis
--physics = { dt = dt, rho0 = 1000}
--vel_phy = dx * velMax / dt

-- Analytical solutions
function wssA(x, y, z)
  return viscPhy * 2 * math.abs(y) * velMaxPhy / R / R
end
function strainRateA(x, y, z)
  return -1*(y) * velMaxPhy / R / R
end
function velA(x, y, z)
  return velMaxPhy * ( 1 - (y)^2 / R^2 )
end

