require "common"

simulation_name = 'channel2D'
debug_mode = false

time = {useIterations = true,
        max = tmaxLB, interval = tmax/10}
estart = {
  read = 'restart/level'..level..'/'..simulation_name..'_lastHeader.lua',
  --write = 'restart/level'..level..'/', 
  --time = { interval  = 1, min = tmax, max = tmax,}
}

add_variable = {
  { name='vel_an', ncomponents=1, vel_an=velA },
  { name='wss_an', ncomponents=1, wss_an=wssA },
  { name='strainRate_an', ncomponents=1, strainRate_an=strainRateA }
}

tracking = {
{  label = 'pressureDrop',
  variable = {
    { 'pressure_phy'},
  },
  shape = { kind = 'canoND', object = {origin = { -length/4, 0.0, dx/2 },
                                       vec = { length/2, 0.0, 0.0},
                                       segments = {200} } },
  time = { min = tmax, max = tmax, interval = 10 },
  folder = './tracking/',
  format = 'asciiSpatial',},
{ label = 'strainRate_spatial',
  variable = {
    {'strainRate_phy'},
    {'strainRate_an'},
    --{name = 'difference', ncomponents = 1, dep = {'wss','wss_an'},},
              },
  shape = { kind = 'canoND',
            object = {origin = { 0.0, -height/2, dx/2},
                      vec = { 0.0, height, 0.0},
                      segments = {200} }},
  time = { min = tmax, max = tmax, interval = 10 },
  folder = './tracking/',
  format = 'asciiSpatial',},
{ label = 'wss_spatial',
  variable = {
    {'wss_phy'},
    {'wss_an'},
    {name = 'difference', ncomponents = 1, dep = {'wss_phy','wss_an'},},
              },
  shape = { kind = 'canoND',
            object = {origin = { 0.0, -height/2, dx/2},
                      vec = { 0.0, height, 0.0},
                      segments = {200} }},
  time = { min = tmax, max = tmax, interval = 10 },
  folder = './tracking/',
  format = 'asciiSpatial',},

{ label = 'wss_l2norm',
  variable = {
    {name = 'difference', ncomponents = 1, dep = {'wss_phy','wss_an'},},
    {'wss_an'},
              },
  shape = { kind = 'canoND',
            object = {origin = { 0.0, -height/2, dx/2},
                      vec = { 0.0, height, 0.0},
                      segments = {100} }},
  time = { min = tmax, max = tmax, interval = 100 },
  reduction = {'l2norm', 'l2norm'},
  folder = './tracking/',
  format = 'asciiSpatial',},

{ label = 'vel_spatial',
  variable = {
    {'velMag_phy'},
    {'vel_an'},
    {name = 'difference', ncomponents = 1, dep = {'velMag_phy','vel_an'},},
              },
  shape = { kind = 'canoND',
            object = {origin = { 0.0, -height/2, dx/2},
                      vec = { 0.0, height, 0.0},
                      segments = {200} }},
  time = { min = tmax, max = tmax, interval = 10 },
  folder = './tracking/',
  format = 'asciiSpatial',},

{ label = 'vel_l2norm',
  variable = {
    {name = 'difference', ncomponents = 1, dep = {'velMag_phy','vel_an'},},
    {'vel_an'},
              },
  shape = { kind = 'canoND',
            object = {origin = { 0.0, -height/2, dx/2},
                      vec = { 0.0, height, 0.0},
                      segments = {100} }},
  time = { min = tmax, max = tmax, interval = 100 },
  reduction = {'l2norm', 'l2norm'},
  folder = './tracking/',
  format = 'asciiSpatial',},
}   -- trakcing table

fluid = { omega = omega, rho0 = densityPhy }
physics = {
  dt = dt,
  rho0 = densityPhy,
} -- physics table

initial_condition = {
  pressure = p0Phy,
  velocityX = 0, velocityY = 0.0, velocityZ = 0.0,
  Sxx = 0, Syy = 0, Szz = 0, Sxy = 0, Syz = 0, Sxz = 0 }
identify = {label = 'level'..level, kind = identify_kind, layout = 'd3q19'}

boundary_condition = {
{ label = 'outlet',  kind = 'outlet_expol', pressure = p0Phy, },
{ label = 'top_wall', kind = 'wall'},
{ label = 'bottom_wall', kind = 'wall'},
                    } -- boundary table
if usePeriodic then
  table.insert( boundary_condition,
        { label = 'inlet',   kind = 'inlet_ubb',
          velocityX = {
            kind = 'combined',
            transient= {
              predefined='smooth',
              min_factor=0.0, max_factor=velMaxPhy, from_time=0, to_time=tmaxPhy/4},
            spatial = {
              predefined='parabol',
              shape = { kind = 'canoND', 
                        object = { origin = { -length/2+dx, -R, dx/2},
                                   vec = { 0.0, 2*R, 0.0 }
                                 }
                      } -- shape table
                      } -- spatial table
                      }, -- velX table
          velocityY = 0.0,
          velocityZ = 0.0,
        })
else
  table.insert( boundary_condition,
        { label = 'left',   kind = 'inlet_ubb',
          velocityX = {
            kind = 'combined',
            transient= {
              predefined='smooth',
              min_factor=0.0, max_factor=velMaxPhy, from_time=0, to_time=tmaxPhy/4},
            spatial = {
              predefined='parabol',
              shape = { kind = 'canoND', 
                        object = { origin = { -length/2+dx, -R*dx, R*dx},
                                   vec = { {0.0, 2*R*dx, 0.0}, {0.0, 0.0, R*dx} }
                                 }
                      } -- shape table
                      } -- spatial table
                      }, -- velX table
          velocityY = 0.0,
          velocityZ = 0.0,
        })
  table.insert( boundary_condition,
      { label = 'front', kind = 'wall' } )
  table.insert( boundary_condition,
      { label = 'back', kind = 'wall' } )
end
