# Run Musubi with different number of processes and check the restart file, 
# if it is identical by using the checksum (md5sum)
# We run the code for a set of defined processes (procs)
# and check the md5sum of the restart files
# by default this script checks last restart write and read files 
# i.e files written at tmax in config file

#------------------------------------------------------------------------#
#  This file is used for recheck. Please make sure that the modifications
#  you make to use it locally, does not get pushed to the repository
#------------------------------------------------------------------------#


# add path to exec if runninng locally. For recheck leave it to "EXEC".
test=0
if [ $test -eq 1 ]; then
  # Test settings
  seeder=seeder
  seeder=../../../../seeder/build/seeder
  executable=../../../build/debug/musubi
  executable=../../../build/musubi
else
  # Recheck settings
  seeder=../../../../seeder/build/seeder
  executable=EXEC
fi

error_flag=0

restart_folder='restart/'
restart_Readfolder='restart_read/'

# delete existing restart folders
rm -rf  ${restart_folder} ${restart_Readfolder}

string=''
do_read=0
debug=1
delFiles=0

procs='1 2 3 4 5 6 7 8 9 10 11 12 13 33 35 36 66 67'
procs='1 2 3'
procs='1 2 3 7 8 17'
# Fails for 18, 19 procs!

if [ $debug -eq 1 ]; then
  rm -rf debug_*
  rm -rf output*
  mkdir output
fi
for refine in 1; do
  export refine=$refine
  for simcase in 'periodic' 'nonperiodic'; do # 'nonperiodic'; do
    export simcase=$simcase
    $seeder

    echo ''
    echo ' Performing test case ' $simcase
    echo ''

    counter=0
    # execute for all procs
    for nprocs in $procs;
    do
      # generate restart WRITE md5 sums
      counter=$((counter+1))
      mkdir -p ${restart_folder}
      export mode=write 
      mpirun -np ${nprocs} ${executable}
      error_flag=$?
      echo 'the result is' $error_flag
      if [ $error_flag -eq 0 ]; then
        echo " SUCCESS "
      else
          echo Return status: $error_flag
          exit $error_flag
      fi
      #file name to store md5sum output from restart write
      sumname=write_${nprocs}.md5
      if [ $debug -eq 1 ]; then
        mkdir -p debug_${nprocs}
#        mv dep*.out dbg*.out debug_${nprocs}
        # This would move the files if they exist
        find . -name "dep*.out" | xargs -i mv {} debug_${nprocs}
        find . -name "dbg*.out" | xargs -i mv {} debug_${nprocs}
#        mv output output_${nprocs}
        find . -name "output" | xargs -i mv {} output_${nprocs}
        mkdir output
      fi  

      echo 'Storing restart write md5sum to ' $sumname
      md5sum ${restart_folder}/*.lsb > ${sumname}
      #md5sum ${restart_folder}/*.lsb | awk '{print $1}' | tail -1 > ${sumname}
      #mv fort.66 write_${nprocs}.out
      

      if [ $counter -eq 1 ]; then
        echo 'setting restart write reference MD5sum'
        firstwritesum=$sumname
      #  tail -1 $sumname | awk '{print $1}' > ref_lastLine
        cut -c 1-32 ${sumname} > ref_lastLine
      fi
      
      if [ $do_read -eq 1 ]; then
      # generate restart READ md5 sums
      mkdir -p  ${restart_Readfolder}
      export mode=read
      mpirun -np ${nprocs} ${executable}
      error_flag=$?
      echo 'the result is' $error_flag
      if [ $error_flag -eq 0 ]; then
        echo " SUCCESS "
      else
          echo Return status: $error_flag
          exit $error_flag
      fi
      #file name to store md5sum output from restart read
      sumname=read_${nprocs}.md5

      echo 'Storing restart read md5sum to '$sumname
      md5sum ${restart_Readfolder}/*.lsb > ${sumname}
      #mv fort.66 read_${nprocs}.out

      # Delete folders
      #rm -rf ${restart_Readfolder}
      #rm -rf ${restart_folder}
      if [ $counter -eq 1 ]; then
        echo 'setting restart read reference MD5sum'
        firstreadsum=$sumname
      fi
      fi
    done

    echo 'testing the difference between the restart write from one proc with more proc'
    counter=0
    for nprocs in $procs;
    do
      counter=$((counter+1))
      string="${firstwritesum} and write_${nprocs}.md5"
      
    #  tail -1 write_${nprocs}.md5 | awk '{print $1}' > lastLine 
      cut -c 1-32 write_${nprocs}.md5 > lastLine 

      if [ $counter -eq 1 ]; then
        echo 'restart write from one proc is set as a reference md5sum:' ${firstwritesum}
      else
        echo  "Now testing difference in the last line between $string (WRITE 1Proc vs. WRITE ${nprocs}) :"
        difference=`diff ref_lastLine lastLine`
        error_flag=$?
        if [ $error_flag -ne 0 ]; then
          echo "ERROR in DIFF"
          exit $error_flag
        fi
        if [ "X$difference" = "X" ]; then
          echo "No difference found. $(tput setaf 2)OK$(tput sgr0)"
        else
          echo "$(tput setaf 1)Error$(tput sgr0): Difference found between ${nprocs} and 1 proc:"
               # md5sum error
               error_flag=99
          echo ${difference}
        fi
      fi
    done
    echo
    echo
    if [ $do_read -eq 1 ]; then
    echo 'testing the difference between the restart write from one proc with restart read-write with one and more proc'
    counter=0
    for nprocs in $procs;
    do
      counter=$((counter+1))

      string="${firstwritesum} and read_${nprocs}.md5"

      tail -1 read_${nprocs}.md5 | awk '{print $1}' > lastLine 
    #  if [ $counter -eq 1 ]; then
    #    echo 'reference md5sum'
    #  else
      echo  "Now testing difference in the last line between ${string} (READ {$nprocs} vs. WRITE 1proc) :"
      difference=`diff ref_lastLine lastLine`
      if [ "X$difference" = "X" ]; then
        echo "No difference found. $(tput setaf 2)OK$(tput sgr0)"
      else
        echo "$(tput setaf 1)Error$(tput sgr0): Difference found:"
        error_flag=99
        echo ${difference}
      fi
    #  fi
    done
    echo 
    fi
    echo 
    echo "Done.        with errorflag" $error_flag
    if [ $delFiles -eq 1 ]; then
      rm ref_lastLine lastLine read_* write_*
    fi 
    if [ $error_flag -ne 0 ]; then
      echo 'Err: exiting with error flag' $error_flag
      exit $error_flag
    fi
  done # simcase loop
done # refine loop
echo 'exiting with error flag' $error_flag
exit $error_flag
