-------------------------------------------------------------------------------
-- This is a sample configuration file to illustrate the general options
-- available via the treelm library.
-- It is supposed to be included in the dependent project configuration files
-- and should always work and represent the current status of the code.
-------------------------------------------------------------------------------

-- The io_buffer_size specifies the amount of memory reserved for blocks of
-- temporary memory allocated for copies of data in MB.
-- (You do not have to specify it at all).
io_buffer_size = 1 -- default is 8 MB

-- Communication pattern to use in parallel executions
commpattern = 'isend_irecv' -- default is simple isend-irecv exchange
--          = 'isend_irecv_overlap' allow overlapping of isends and irecvs
--          = 'typed_isend_irecv' use a MPI datatype to describe the data
--          = 'gathered_type' minimized number of blocks

-- Print memory upon finalize?
-- This is obtained from the Linux pseudo file /proc/self/status.
-- It is only printed by the first process in a prallel run, and only provides
-- data on this first process.
printRuntimeInfo = true --default is true

-- Settings for the primary logging object provided by treelm.
-- (Root outputs to stdout)
-- The main control here is the level, which tells the logger how verbose
-- the output should be. The higher the log level, the more verbose your
-- output will be. Default is 1.
-- Use 0 to deactivate log messages in the primary logger.
-- Further settings are real_form to define the format of real numbers
-- (default = 'EN12.3') and int_form to define the format of integer
-- numbers (default = 'I0')
logging = { level = 1 }

--------------------------------------------------------------------------------
-- MESH DEFINITION
-- Defining the mesh can take two forms:
-- 1) simply a string specifying the directory to read the mesh from, if it is
--    indeed a directory there has to be a trailing "/", otherwise the string is
--    simply a prefix to all mesh files.
--mesh = './mesh/'       -- (default = 'mesh/')
--
-- 2) a table for a predefined mesh. Right now only the full 'cube' without any
--    boundary conditions (periodic in all directions) is available as a
--    predefined mesh.
mesh = {
        predefined = 'cube',      -- use the predefined full cube
        origin = {                -- origin of the cube
                   (-1.0)*cube_length/2.0,
                   (-1.0)*cube_length/2.0,
                   (-1.0)*cube_length/2.0
                 },
        length = cube_length,     -- length of the cube
        refinementLevel = 4       -- refinement level to resolve the cube
       }
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- DEBUG OPTIONS
--debug = { 
--         debugMode = true,        -- default= false
--         verbose = 3,             -- default= 0

--         -- Use a logging object to track output in individual files.
--         -- The output messages to this logging object are usually
--         -- generated by tem_debug calls.
--         logging = { level = 1, -- how detailed should the log be?
--                     filename = 'dbg', -- to which file to write the log
--                     root_only = false }, -- should only root write msgs?

--         debugFiles = true,       -- default= false
--         -- What to dump into debugFiles? --
--           dumpTreeIDs = true,      -- default= false
--           dumpPropBits = true,     -- default= false
--           dumpAuxLists = true,     -- default= false
--           dumpDependencies = true, -- default= false
--           dumpState = true,        -- default= false
--           dumpHaloState = true,    -- default= false
--         --  end debugFiles  --
--         debugDependencies = true, -- default= false
--         checkDependencies = true, -- default= false
--         checkNans = true,         -- default= false
--         checkSteps = true,        -- default= false
--         debugMesh = 'dbg/mesh_',  -- default= ''
--         debugSource = true,       -- default= false
--         debugRestart = true,      -- default= false
--         traceMemory = true,       -- default= false
--        }
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- RESTART SETTINGS
-- useIteration = true: restart files are written at the defined numbers of Iteration.
--                      min, max and interval have to be given as integers.
-- useIteration = false: restart files are written when the specified time is reached.
--                       min, max and interval have to given as reals.
-- min: minimal timestep to start from ,
-- max: maximal timestep to reach,
-- interval is the interval of writing restart files between min and max.
-- in addition, one restart file is written at 0.0 and one at the final time of the simulation.

restart = {
        --   read = './restart/sample_lastHeader.lua',  -- header of restart data to read
           write = 'restart_',                          -- prefix of to name restart files
        --   write = 'restart/',                        -- if this ends with a path separator
        --                                                 restart data will be written into
        --                                                 this directory, which has to exist!
           time = { useIterations = false, min = 0, max = fintime, interval = fintime }
                -- (default: useIteration = true, min = 0, max = 0, interval = 1)
          }
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- TRACKING
--
-- tracking = {
--             { label = 'label',                -- (default = 'unnamed')
--               variable = {{'density'}, {'momentum'}},
--               -- transient_reduction = {'sum','average'}, -- (default = 'none')
--               -- reduction = {'sum','average'},  -- (default = 'none')
--               shape = { kind = 'canoND', object = {origin = {centerX,
--                                                              centerY,
--                                                              centerZ}
--                                                    }
--                       },
--               time = {useIterations = false,
--                       min = 0, max = endtime,
--                       interval = interval},
--               format = 'harvester',               -- (default = 'harvester')
--               folder = './tracking/'          --(default = './')
--             }
--           }

--------------------------------------------------------------------------------
-- TRANSIENT_REDUCTION
-- This is a reduction in time, so every value at every point in the tracking area is replaced in each timestep
-- by the result of a function that has probably access to data from previous timesteps.
-- This "transient-reduction" is applied before any spatial "reduction" operation.

-- REDUCTION
-- reduction = {'sum','average', 'l2norm', 'max', 'min', 'none'},
-- If we track more than one variable, there are two options to specify the reduction:
-- 1) give one reduction for all variables
-- 2) give as many reductions as there are are variables, the first reduction belongs to the first variable,
--    the second reduction to the second variable and so on.

-- SHAPE

-- Point
-- shape = { kind = 'canoND', object = {origin = {0.0, 0.0, 0.0} } }

-- Line 
-- shape = { kind = 'canoND', object = {origin = {0, 0, 0}, vec = {0, 10, 0},
--                  segments = {100}, distribution = 'equal' } }

-- Plane 
-- shape = { kind = 'canoND', object = {origin = {0, 0, 0}, vec = {{10, 0, 0}, {0, 10, 0}}, 
--                  segments = {100, 100}, distribution = 'equal' } }

-- Box 
-- shape = { kind = 'canoND', object = {origin = {0, 0, 0}, vec = {{10, 0, 0}, {0, 10, 0}, {0, 0, 10}}, 
--                  segments = {100, 100, 100}, distribution = 'equal' } }
-- shape = { kind = 'canoND', object = {origin = {0, 0, 0}, length = 10, 
--                  segments = 100, distribution = ‘equal' } },

-- FORMAT
-- format = 'ascii',
-- format = 'asciiSpatial',
-- format = 'harvester',
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- POINT / LINE / PLANE DEFINITION

-- Point
-- shape = { kind = 'canoND', object = {origin = {0.0, 0.0, 0.0} } }

-- Line 
-- shape = { kind = 'canoND', object = {origin = {0, 0, 0}, vec = {0, 10, 0},
--                  segments = {100}, distribution = 'equal' } }

-- Plane 
-- shape = { kind = 'canoND', object = {origin = {0, 0, 0}, vec = {{10, 0, 0}, {0, 10, 0}}, 
--                  segments = {100, 100}, distribution = 'equal' } }

-- Box 
-- shape = { kind = 'canoND', object = {origin = {0, 0, 0}, vec = {{10, 0, 0}, {0, 10, 0}, {0, 0, 10}}, 
--                  segments = {100, 100, 100}, distribution = 'equal' } }
-- shape = { kind = 'canoND', object = {origin = {0, 0, 0}, length = 10, 
--                  segments = 100, distribution = ‘equal' } },
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- PREDEFINED SPATIAL
---- gauss pulse
--{predefined = 'gausspulse',
--              center = { 5.0, 5.0, 5.0 },    -- Gauss pulse center
--              halfwidth = 2.0,               -- half width of gauss pulse from center
--              amplitude = 2.0,               -- height or magnitude of gauss pulse
--              background = 1.225 }           -- reference value. In case of density, it is reference density
--
---- 2d co-rotating vortex pair
--{predefined = 'crvpX',                       -- or 'crvpY'
--              center = { 5.0, 5.0, 5.0 },    -- spinning center
--              radius_rot = 10.0,             -- distance of vortex centers / 2
--              circulation = 2.0 }            -- circulation of vortices
--
---- parabol 2d
--{predefined = 'parabol',
--               shape = { kind = 'canoND', 
--                         object = {origin = {0, 0, 0}, 
--                                   vec = {0, 10, 0},
--                                  }
--                       }
--}
--
---- parabol 3d
--{predefined = 'parabol',
--               shape = { kind = 'canoND', 
--                         object = {origin = {0, 0, 0}, 
--                                   vec = {{10, 0, 0}, {0, 10, 0}}, 
--                                   }
--                       }
--}
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- PREDEFINED TRANSIENT
---- smooth sin function
--{ predefined = 'smooth',
--  min_factor = 0.0,
--  max_factor = 1.0,
--  from_time = 0,
--  to_time = 1.0
--}
--
---- linear function
--{ predefined = 'linear',
--  min_factor = 0.0,
--  max_factor = 1.0,
--  from_time = 0,
--  to_time = 1.0
--}

----EXAMPLE
--
---- in boundaries:
--velocityX = { ref_value = 100,
--              transient = { predefined = 'smooth',  -- smooth sin function
--                            min_factor = 0.0,
--                            max_factor = 1.0,
--                            from_time = 0,
--                            to_time = 1.0
--                          }
--            }
--
---- in initial conditions:
--density = { predefined = 'gausspulse',
--            center = { 5.0, 5.0, 5.0 },    -- Gauss pulse center
--            halfwidth = 2.0,               -- half width of gauss pulse from center
--            amplitude = 2.0,               -- height or magnitude of gauss pulse
--            backgound = 1.225 }   -- reference value. In case of density, it is reference density
--          }

--------------------------------------------------------------------------------
-- PREDEFINED SPACETIME

---- constant for all (x,y,z,t)
--spacetime = 5.0
--
---- combined, returns spatial(x,y,z)*transient(t)
--spacetime = {
--              spatial = { predefined = 'gausspulse',
--                                        center = { 5.0, 5.0, 5.0 },    -- Gauss pulse center
--                                        halfwidth = 2.0,               -- half width of gauss pulse from center
--                                        amplitude = 2.0,               -- height or magnitude of gauss pulse
--                                        backgound = 1.225
--                         },
--              transient = { predefined = 'smooth',  -- smooth sin function
--                                         min_factor = 0.0,
--                                         max_factor = 100.0,
--                                         from_time = 0,
--                                         to_time = 1.0
--                          }
--            }
--
---- lua function
-- Space-Time functions can be expected to return scalars or 1d arrays.
-- Scalars are just single values that are returned, while arrays have
-- to be tables that are returned.
-- Note, that whenever there is only a single value expected, a scalar
-- should be returned, not a table with a single value!
--spacetime = luafunction
--------------------------------------------------------------------------------
