module tem_precice_module
  use env_module,            only: rk, labelLen
  use tem_time_module,       only: tem_time_type
  use aotus_module,          only: flu_State

  implicit none

  private


  logical , parameter :: precice_available= .false.

  type tem_precice_type
  ! general info for precice
  ! name of solver, should be indetical to name in precice_configeFile
  character(len=labelLen) :: accessor
  character(len=labelLen) :: precice_configFile
  character(len=labelLen) :: mesh_name
  integer          :: meshID
  !> for reading data
  integer          :: n_readData=0 !--> length of datatable in Lua
  character(len=labelLen), allocatable :: read_data_name(:)
  integer, allocatable          :: read_dataID(:)
  !> for writing data
  integer          :: n_writeData=0 !--> length of datatable in Lua
  character(len=labelLen), allocatable :: write_data_name(:)
  integer, allocatable          :: write_dataID(:)
  !> Number of read positions ==> number of gauss points at bc 
  integer          :: n_read_points=0
  !> Number of write positions ==> number of gauss points at bc 
  integer          :: n_write_points=0
  end type tem_precice_type

  type(tem_precice_type) :: precice

  interface tem_spacetime_precice_for
    module procedure tem_spacetime_precice_scalar
    module procedure tem_spacetime_precice_vector
  end interface tem_spacetime_precice_for

  interface tem_precice_write
    module procedure tem_precice_write_scalar
    module procedure tem_precice_write_vector
  end interface tem_precice_write

  public :: tem_precice_load, tem_precice_init, tem_precice_create
  public :: tem_precice_advance
  public :: tem_precice_finalize
  public :: tem_precicef_ongoing
  public :: tem_spacetime_precice_for
  public :: tem_precice_write
  public :: tem_precicef_set_read_pos
  public :: tem_precicef_set_write_pos
  public :: precice
  public :: precice_available

contains

  subroutine tem_precice_load(conf)
    !---------------------------------------------------------------------------
    !> The filename to read the configuration from.
    type(flu_state), intent(in) :: conf
  end subroutine tem_precice_load
  subroutine tem_precice_create
  end subroutine tem_precice_create

  subroutine tem_precice_init(timestepLimit)
    !---------------------------------------------------------------------------
    real(kind=rk), intent(inout) :: timestepLimit

  end subroutine tem_precice_init

  subroutine tem_precice_advance(timestepLimit)
    !---------------------------------------------------------------------------
    real(kind=rk), intent(inout) :: timestepLimit
  end subroutine tem_precice_advance


  subroutine tem_precice_finalize()

  end subroutine tem_precice_finalize
  
  subroutine tem_precicef_ongoing(isOngoing)
    !---------------------------------------------------------------------------
    integer, intent(inout)   :: isOngoing
  end subroutine tem_precicef_ongoing

  subroutine tem_precicef_set_read_pos(position, positionID )
    !---------------------------------------------------------------------------
    real(kind=rk), intent(in) :: position(3)
    integer, intent(out) :: positionID  
  endsubroutine tem_precicef_set_read_pos


  subroutine tem_precicef_set_write_pos(position, positionID )
    !---------------------------------------------------------------------------
    real(kind=rk), intent(in) :: position(3)
    integer, intent(out) :: positionID  
    !---------------------------------------------------------------------------

  endsubroutine tem_precicef_set_write_pos
  function tem_spacetime_precice_scalar(dataID, posID, npoints) result(res)
  !------------------------------------------------------------------------
    integer, intent(in)          :: dataID
    integer, intent(in)          :: posID(:)
    integer, intent(in)          :: npoints
    real(kind=rk) :: res(npoints) !< return value
    res = 0.0_rk
  end function tem_spacetime_precice_scalar

  function tem_spacetime_precice_vector(dataID, posID, npoints, vector) result(res)
    !------------------------------------------------------------------------
    integer, intent(in)          :: dataID
    integer, intent(in)          :: posID(:)
   ! character(len=*), intent(in) :: dataName
    integer, intent(in)          :: npoints
    logical, intent(in)          :: vector
    real(kind=rk) :: res(npoints,3) !< return value
    !------------------------------------------------------------------------

  end function tem_spacetime_precice_vector
!******************************************************************************!

  subroutine tem_precice_write_scalar(dataID, posID, npoints, val) 
    !------------------------------------------------------------------------
    integer, intent(in)          :: dataID
    integer, intent(in)          :: posID(npoints)
    integer, intent(in)          :: npoints
    real(kind=rk), intent(in)    :: val(npoints)
    !------------------------------------------------------------------------
  end subroutine tem_precice_write_scalar

  subroutine tem_precice_write_vector(dataID, posID, npoints, val) 
    !------------------------------------------------------------------------
    integer, intent(in)          :: dataID
    integer, intent(in)          :: posID(:)
    integer, intent(in)          :: npoints
    real(kind=rk), intent(in)    :: val(3,npoints)
  end subroutine tem_precice_write_vector
end module tem_precice_module
