! See copyright notice in the COPYRIGHT file.
!******************************************************************************!
!> Module to provide access to the gamma function.
!!
!! This dummy module is used, if the compiler does not
!! support this F2008 feature, and the NAG implementation
!! is not found.
!! We fallback to the gammln function from the numerical
!! recipes here.
!!
module tem_gamma_module

  ! include treelm modules
  use env_module, only: rk

  implicit none

  private

  logical, parameter, public :: elemental_gamma = .true.

  real(kind=rk), parameter :: stp = 2.5066282746310005_rk
  real(kind=rk), parameter :: cof(6) = [                  &
    &                          76.18009172947146_rk     , &
    &                         -86.50532032941677_rk     , &
    &                          24.01409824083091_rk     , &
    &                          -1.231739572450155_rk    , &
    &                            .1208650973866179e-2_rk, &
    &                           -.5395239384953e-5_rk     ]

  public :: gamma

  contains

!******************************************************************************!
  !> todo: Add description
  !!
  elemental function gammln(xx)
    !---------------------------------------------------------------------------
    real(kind=rk),intent(in) :: xx
    real(kind=rk) :: gammln
    !---------------------------------------------------------------------------
    integer :: j
    real(kind=rk) :: ser, tmp, x, y
    !---------------------------------------------------------------------------

    x   = xx
    y   = x
    tmp = x+5.5_rk
    tmp = (x+0.5_rk)*log(tmp) - tmp
    ser = 1.000000000190015_rk

    do  j=1,6
       y  = y+1.0_rk
       ser= ser+cof(j)/y
    end do

    gammln = tmp + log(stp*ser/x)

  end function gammln
!******************************************************************************!


!******************************************************************************!
  !> todo: Add description
  !!
  elemental function gamma(xx)
    !---------------------------------------------------------------------------
    real(kind=rk),intent(in) :: xx
    real(kind=rk) :: gamma
    !---------------------------------------------------------------------------

    gamma = exp(gammln(xx))

  end function gamma
!******************************************************************************!


end module tem_gamma_module
!******************************************************************************!
