! See copyright notice in the COPYRIGHT file.
!******************************************************************************!
!> This module provides basic information on the parallel environment
module tem_comm_env_module

  ! include treelm modules
  use mpi

  implicit none

  private

  public :: tem_comm_env_type
  public :: process
  public :: tem_comm_env_init, tem_comm_env_fin
  public :: tem_comm_env_init_empty

  !> Information about parallel runs
  type tem_comm_env_type
    !> size of MPI communicator
    integer :: comm_size
    !> MPI rank
    integer :: rank
    !> MPI root rank
    integer :: root
    !> MPI communicator
    integer :: comm
  end type tem_comm_env_type

  !> \todo HK: should NOT be a module variable, as we might
  !!   want to have different communicators in the application...
  type(tem_comm_env_type) :: process

  contains

!******************************************************************************!
  !> Initialize the environment. This routine is called by tem_start
  !! which should be the very first action in a program.
  !!
  subroutine tem_comm_env_init( proc )
    !--------------------------------------------------------------------------=
    !> The process communicator type
    type( tem_comm_env_type ), optional :: proc
    !---------------------------------------------------------------------------
    !> Error flag
    integer :: iError
    !--------------------------------------------------------------------------

    if( present( proc ) ) then
      ! Init MPI rank, size and root
      call mpi_comm_dup( mpi_comm_world, proc%comm, iError )
      call mpi_comm_rank( proc%comm, proc%rank, iError )
      call mpi_comm_size( proc%comm, proc%comm_size, iError )
      proc%root = 0
    else
      ! Init MPI rank, size and root
      call mpi_comm_dup( mpi_comm_world, process%comm, iError )
      call mpi_comm_rank( process%comm, process%rank, iError )
      call mpi_comm_size( process%comm, process%comm_size, iError )
      process%root = 0
    endif

  end subroutine tem_comm_env_init
!******************************************************************************!


!******************************************************************************!
  !> Initialize a debug environment for just one process without envoking MPI
  !!
  subroutine tem_comm_env_init_empty( proc )
    !---------------------------------------------------------------------------
    !> The process communicator type
    type( tem_comm_env_type ), optional :: proc
    !---------------------------------------------------------------------------

    if( present( proc ) ) then
      ! Init MPI rank, size and root
      proc%comm = 0
      proc%rank = 0
      proc%comm_size = 0
      proc%root = 0
    else
      ! Init MPI rank, size and root
      process%comm = 0
      process%rank = 0
      process%comm_size = 0
      process%root = 0
    endif

  end subroutine tem_comm_env_init_empty
!******************************************************************************!


!******************************************************************************!
  !> Finalize the environment. This routine is called by tem_finalize
  !! which should be the very last action in a program.
  !!
  subroutine tem_comm_env_fin( proc )
    !---------------------------------------------------------------------------
    !> The process communicator type
    type( tem_comm_env_type ), optional :: proc
    !---------------------------------------------------------------------------
    !> Error flag
    integer :: iError
    !---------------------------------------------------------------------------

    ! Free the communicator again
    if( present( proc ) ) then
      call mpi_comm_free( proc%comm, iError )
    else
      call mpi_comm_free( process%comm, iError )
    endif

  end subroutine tem_comm_env_fin
!******************************************************************************!


end module tem_comm_env_module
!******************************************************************************!
