! See copyright notice in the COPYRIGHT file.
!******************************************************************************!
!> Initialize general infrastructure settings of treelm.
!!
!! \author Harald Klimach
!!
!! This module provides a central point to initialize the
!! various general settings in treelm.
!!
module tem_general_module

  ! include aotus modules
  use aotus_module, only: flu_State

  ! include treelm modules
  use env_module,         only: tem_load_env_params
  use tem_logging_module, only: tem_logging_load_primary
  use tem_comm_module,    only: tem_commpattern_type, tem_load_commpattern
  use tem_debug_module,   only: tem_debug_load_main
  use tem_tools_module,   only: tem_write, upper_to_lower

  use tem_precice_module, only: precice_available, tem_precice_load

  implicit none

  private

  public :: tem_general_load

  contains

!******************************************************************************!
  !> Load general treelm settings from the Lua script in conf.
  !!
  subroutine tem_general_load( conf, rank, commpattern )
    !---------------------------------------------------------------------------
    !> Handle to the Lua script containing the configuration.
    type(flu_state) :: conf
    !> Rank of the calling process.
    integer, intent(in) :: rank
    !> Communication pattern to use.
    type(tem_commPattern_type), intent(out) :: commpattern
    !---------------------------------------------------------------------------

    call tem_load_env_params(conf)
    call tem_logging_load_primary(conf = conf, rank = rank)
    call tem_debug_load_main(conf = conf, rank = rank)
    call tem_load_commpattern(conf = conf, me = commpattern)
    if (precice_available) then
      call tem_write('Loading precice data' )
      call tem_precice_load(conf = conf)
    end if

  end subroutine tem_general_load
!******************************************************************************!
  

end module tem_general_module
!******************************************************************************!
