! See copyright notice in the COPYRIGHT file.
!******************************************************************************!
!> This module  provides descriptions for elements
!!
?? include 'arrayMacros.inc'
module tem_halo_module

  ! include treelm modules
  use env_module,           only: minLength, zeroLength
  use tem_dyn_array_module, only: dyn_intArray_type, init, append, expand,     &
    &                             destroy, SortedPosOfVal, PositionOfVal

  implicit none

  private

  public :: tem_halo_type
  ! dynamic array of variable types
  public :: dyn_haloArray_type, init, destroy, append, expand, positionofval,  &
    &       sortedposofval
  public :: operator(==), operator(/=)
  public :: operator(<), operator(<=), operator(>), operator(>=)
  public :: assignment(=)

  type tem_halo_type
    !> process number of this partner process
    integer :: partnerProc
    !> list of my halo elements, which I request from sourceProc
    type(dyn_intArray_type) :: mine
  end type tem_halo_type

! declare the dynamic array of element type
?? copy :: DA_decltxt(halo, type(tem_halo_type))

  interface assignment(=)
    module procedure Copy_var
  end interface

  interface operator(==)
    module procedure isEqual
  end interface

  interface operator(/=)
    module procedure isUnequal
  end interface

  interface operator(<)
    module procedure isSmaller
  end interface

  interface operator(<=)
    module procedure isSmallerOrEqual
  end interface

  interface operator(>)
    module procedure isGreater
  end interface

  interface operator(>=)
    module procedure isGreaterOrEqual
  end interface

  contains

!******************************************************************************!
  !> Include the subroutines for the dynamic array.
?? copy :: DA_impltxt(halo, type(tem_halo_type), type(tem_halo_type))


!******************************************************************************!
  !> This function provides the assignment operator of two variables.
  !! Temporary Solution as CRAY compiler dont have = Operator
  !! Copying a Variable Components into other Variable
  !!
  subroutine Copy_var(left, right)
    !---------------------------------------------------------------------------
    type(tem_halo_type), intent(out) :: left
    type(tem_halo_type), intent(in) :: right
    !---------------------------------------------------------------------------

    ! First compare the string of the variable names
    left%partnerProc   = right%partnerProc
    left%mine         = right%mine
!    left%his          = right%his 
  end subroutine Copy_var
!******************************************************************************!


!******************************************************************************!
  !> This function provides the test for equality of two variables.
  !!
  !! Two variables are considered to be equal, if their name and number of
  !! components are equal.
  !!
  function isEqual(left, right) result(equality)
    !---------------------------------------------------------------------------
    type(tem_halo_type), intent(in) :: left
    type(tem_halo_type), intent(in) :: right
    logical :: equality
    !---------------------------------------------------------------------------

    equality = (left%partnerProc == right%partnerProc) 

  end function isEqual
!******************************************************************************!


!******************************************************************************!
  !> This function provides the test for unequality of two variables.
  !!
  !! Two variables are considered to be equal, if their name and number of
  !! components are equal.
  !!
  function isUnequal(left, right) result(unequality)
    !---------------------------------------------------------------------------
    type(tem_halo_type), intent(in) :: left
    type(tem_halo_type), intent(in) :: right
    logical :: unequality
    !---------------------------------------------------------------------------

    unequality = (left%partnerProc /= right%partnerProc) 

  end function isUnequal
!******************************************************************************!


!******************************************************************************!
  !> This function provides a comparison of two variables.
  !!
  !! A lexical ordering is used for the names, and as second criteria the
  !! number of components are used.
  !!
  function isSmaller(left, right) result(small)
    !---------------------------------------------------------------------------
    type(tem_halo_type), intent(in) :: left
    type(tem_halo_type), intent(in) :: right
    logical :: small
    !---------------------------------------------------------------------------

    small = (left%partnerProc < right%partnerProc) 

  end function isSmaller
!******************************************************************************!


!******************************************************************************!
  !> This function provides a comparison of two variables.
  !!
  !! A lexical ordering is used for the names, and as second criteria the
  !! number of components are used.
  !!
  function isSmallerOrEqual(left, right) result(small)
    !---------------------------------------------------------------------------
    type(tem_halo_type), intent(in) :: left
    type(tem_halo_type), intent(in) :: right
    logical :: small
    !---------------------------------------------------------------------------

    small = (left%partnerProc <= right%partnerProc) 

  end function isSmallerOrEqual
!******************************************************************************!


!******************************************************************************!
  !> This function provides a comparison of two variables.
  !!
  !! A lexical ordering is used for the names, and as second criteria the
  !! number of components are used.
  !!
  function isGreater(left, right) result(great)
    !---------------------------------------------------------------------------
    type(tem_halo_type), intent(in) :: left
    type(tem_halo_type), intent(in) :: right
    logical :: great
    !---------------------------------------------------------------------------

    great = (left%partnerProc > right%partnerProc) 

  end function isGreater
!******************************************************************************!


!******************************************************************************!
  !> This function provides a comparison of two variables.
  !!
  !! A lexical ordering is used for the names, and as second criteria the
  !! number of components are used.
  !!
  function isGreaterOrEqual(left, right) result(great)
    !---------------------------------------------------------------------------
    type(tem_halo_type), intent(in) :: left
    type(tem_halo_type), intent(in) :: right
    logical :: great
    !---------------------------------------------------------------------------

    great = (left%partnerProc >= right%partnerProc) 

  end function isGreaterOrEqual
!******************************************************************************!


end module tem_halo_module
!******************************************************************************!
