! See copyright notice in the COPYRIGHT file.
!******************************************************************************!
!> This module handles the various status bits and performs the relevant
!! communications. For example if one process ends, this information is stored
!! in a bit and communicated to all other processors.
!!
!! \author Kartik Jain
!!
module tem_status_module

  ! include treelm modules
  use mpi
  use tem_comm_env_module, only: process

  implicit none

  private

  public :: tem_stat_nFlags
  public :: tem_status_type
  public :: tem_status_communicate
  public :: tem_stat_runEnd, tem_stat_run_terminate, tem_stat_steady_state
  public :: tem_stat_maxWallReached, tem_stat_nan_Detected
  public :: tem_stat_nonPhysical
  public :: tem_stat_global_error
  public :: tem_status_init

  !> Number of available status flags.
  integer, parameter :: tem_stat_nFlags = 7

  ! Definition of the various status flags
  ! Remember to increase the counter of nFlags above if you add a new one.
  integer, parameter :: tem_stat_runEnd           = 1 
  integer, parameter :: tem_stat_run_terminate    = 2
  integer, parameter :: tem_stat_steady_state     = 3
  integer, parameter :: tem_stat_global_error     = 4
  integer, parameter :: tem_stat_nan_detected     = 5
  integer, parameter :: tem_stat_maxWallReached   = 6
  integer, parameter :: tem_stat_nonPhysical      = 7

  !> Define an array to hold all status flags
  type tem_status_type
    logical :: bits(tem_stat_nFlags) = .false.
  end type


contains


!******************************************************************************!
  !> Initialize the status bits
  !!
  subroutine tem_status_init(me)
    !---------------------------------------------------------------------------
    !> status type to initialize
    type(tem_status_type), intent(inout) :: me
    !---------------------------------------------------------------------------
    me%bits = .false.

  end subroutine tem_status_init
!******************************************************************************!


!******************************************************************************!
  !> Perform the communication of status bits
  !!
  subroutine tem_status_communicate(me)
    !---------------------------------------------------------------------------
    !> status type to comunicate
    type(tem_status_type), intent(inout) :: me
    !---------------------------------------------------------------------------
    integer :: iError
    logical :: local_bits(tem_stat_nFlags)
    !---------------------------------------------------------------------------
    local_bits = me%bits
    call mpi_allreduce( local_bits, me%bits, tem_stat_nFlags, MPI_LOGICAL,     &
      &                 MPI_LOR, process%comm, iError )

  end subroutine tem_status_communicate
!******************************************************************************!


end module tem_status_module
!******************************************************************************!
