! See copyright notice in the COPYRIGHT file.
program tem_face_test
  use tem_utestEnv_module,  only: load_env
  use tem_comm_module,      only: tem_commPattern_type  
  use tem_face_module,      only: tem_face_type, tem_build_face_info
  use treelmesh_module,     only: treelmesh_type 
  use tem_bc_prop_module,   only: tem_bc_prop_type
  use env_module,           only: long_k
  use tem_dyn_array_module, only: PositionOfVal 
  use tem_tools_module,     only: tem_write
  use tem_aux_module,       only: tem_abort, tem_finalize

!use tem_element_module, only: PositionOfVal

  implicit none
 
  write(*,*) 'Running tem_construction_test...'
  call check_serial_singlelevel_faceDesc()

contains

  subroutine check_serial_singlelevel_faceDesc()
!    character(len=*) :: filename
    integer i
    integer :: pos
    type(tem_commPattern_type) :: commPattern
    type(treelmesh_type) :: tree
    type(tem_bc_prop_type) :: boundary 
    type(tem_face_type) :: faces(1:4)
 !   type(tem_face_type) :: faces(tree%global%minLevel:tree%global%maxLevel)
      

    ! Load the environment.
    call load_env(tree = tree, boundary = boundary, &
      &           commPattern = commPattern )

    ! Create face description
    call tem_build_face_info( tree = tree, boundary = boundary, &
                            & commPattern = commPattern, &
                            & faces = faces )

    write(*,*) "Hello!!!!!!!"
    write(*,*) tree%nElems

    do i=1, tree%nElems
       write(*,*) tree%treeID(i)
    end do

    write(*,*) tree%part_Last(1)
    write(*,*) tree%elemOffset


    ! Start our checks:
    
    !!!!!!!!!!!!!! Bottom Plane z=0 !!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! We only have single level, so iterate over all these faces.
    ! First, we start with the faces aligned in x direction.
    
    ! Let us have a look if the number of faces is 8.
    call tem_write('X-direction!')
    if( size(faces(1)%faces(1)%computeFace%leftPos).ne.8 ) then
      call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'expected 8 faces, but face description is wrong ...')
      call tem_abort()
    else 
      call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'number of compute faces is 8.')
    end if
     
    ! Let us have a look if the face 2-1 (left elem id - right elem id) is
    ! in the list of faces.
    pos = PositionOfVal( me = faces(1)%faces(1)%faceList%faceId, val = 2_long_k )
    if(pos .le. 0) then
      call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'not able to locate face 2-1 in face list ...')
      call tem_abort()
    else 
      call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'found face 2-1 in face list.')
      if(faces(1)%faces(1)%faceList%rightElemId%val(pos).ne.1_long_k) then
        call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'not able to locate face 2-1 in face list (wrong ' // &
                  & 'right elem id) ...')
        call tem_abort()
      else 
        call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'found face 2-1 in face list, correct right elem id.')
      end if
    end if


    ! Checking in y-direction    
    ! Let us have a look if the number of faces is 8.
    call tem_write('Y-direction')
    if( size(faces(1)%faces(2)%computeFace%leftPos).ne.8 ) then
      call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'expected 8 faces, but face description is wrong ...')
      call tem_abort()
    else 
      call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'number of compute faces is 8.')
    end if
     
    ! Let us have a look if the face 3-1 (left elem id - right elem id) is
    ! in the list of faces.
    pos = PositionOfVal( me = faces(1)%faces(2)%faceList%faceId, val = 3_long_k )
    if(pos .le. 0) then
      call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'not able to locate face 3-1 in face list ...')
      call tem_abort()
    else 
      call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'found face 3-1 in face list.')

      if(faces(1)%faces(2)%faceList%rightElemId%val(pos).ne.1_long_k) then
        call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'not able to locate face 3-1 in face list (wrong ' // &
                  & 'right elem id) ...')
        call tem_abort()
      else 
        call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'found face 3-1 in face list, correct right elem id.')
      end if
    end if




    ! Checking in z-direction    
    ! Let us have a look if the number of faces is 8.
    call tem_write('Z-direction')
    if( size(faces(1)%faces(3)%computeFace%leftPos).ne.8 ) then
      call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'expected 8 faces, but face description is wrong ...')
      call tem_abort()
    else 
      call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'number of compute faces is 8.')
    end if
     
    ! Let us have a look if the face 1-5 (left elem id - right elem id) is
    ! in the list of faces.
    pos = PositionOfVal( me = faces(1)%faces(3)%faceList%faceId, val = 1_long_k )
    if(pos .le. 0) then
      call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'not able to locate face 1-5 in face list ...')
      call tem_abort()
    else 
      call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'found face 1-5 in face list.')

      if(faces(1)%faces(3)%faceList%rightElemId%val(pos).ne.5_long_k) then
        call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'not able to locate face 1-5 in face list (wrong ' // &
                  & 'right elem id) ...')
        call tem_abort()
      else 
        call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'found face 1-5 in face list, correct right elem id.')
      end if
    end if



    ! Checking in z-direction    
    ! Let us have a look if the face 4-8 (left elem id - right elem id) is
    ! in the list of faces.
    pos = PositionOfVal( me = faces(1)%faces(3)%faceList%faceId, val = 4_long_k )
    if(pos .le. 0) then
      call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'not able to locate face 4-8 in face list ...')
      call tem_abort()
    else 
      call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'found face 4-8 in face list.')

      if(faces(1)%faces(3)%faceList%rightElemId%val(pos).ne.8_long_k) then
        call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'not able to locate face 4-8 in face list (wrong ' // &
                  & 'right elem id) ...')
        call tem_abort()
      else 
        call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'found face 4-8 in face list, correct right elem id.')
      end if
    end if


    !!!!!!!!!!!!!!!!! Top plane Z=1 !!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! We only have single level, so iterate over all these faces.
    ! First, we start with the faces aligned in x direction.
    call tem_write('X-direction')
     
    ! Let us have a look if the face 6-5 (left elem id - right elem id) is
    ! in the list of faces.
    pos = PositionOfVal( me = faces(1)%faces(1)%faceList%faceId, val = 6_long_k )
    if(pos .le. 0) then
      call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'not able to locate face 6-5 in face list ...')
      call tem_abort()
    else 
      call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'found face 6-5 in face list.')
      if(faces(1)%faces(1)%faceList%rightElemId%val(pos).ne.5_long_k) then
        call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'not able to locate face 6-5 in face list (wrong ' // &
                  & 'right elem id) ...')
        call tem_abort()
      else 
        call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'found face 6-5 in face list, correct right elem id.')
      end if
    end if


    ! Checking in y-direction    
    ! Let us have a look if the number of faces is 8.
    call tem_write('Y-direction')
    if( size(faces(1)%faces(2)%computeFace%leftPos).ne.8 ) then
      call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'expected 8 faces, but face description is wrong ...')
      call tem_abort()
    else 
      call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'number of compute faces is 8.')
    end if
     
    ! Let us have a look if the face 7-5 (left elem id - right elem id) is
    ! in the list of faces.
    pos = PositionOfVal( me = faces(1)%faces(2)%faceList%faceId, val = 7_long_k )
    if(pos .le. 0) then
      call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'not able to locate face 7-5 in face list ...')
      call tem_abort()
    else 
      call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'found face 7-5 in face list.')

      if(faces(1)%faces(2)%faceList%rightElemId%val(pos).ne.5_long_k) then
        call tem_write('ERROR in check_serial_singlelevel_faceDesc : ' // &
                  & 'not able to locate face 7-5 in face list (wrong ' // &
                  & 'right elem id) ...')
        call tem_abort()
      else 
        call tem_write('SUCCESS in check_serial_singlelevel_faceDesc : ' // &
                  & 'found face 7-5 in face list, correct right elem id.')
      end if
    end if

    call tem_finalize()

    ! When we reach this point, all tests have passed...
    write(*,*) 'PASSED'

    call tem_write('PASSED')

  end subroutine check_serial_singlelevel_faceDesc

end program tem_face_test
