! See copyright notice in the COPYRIGHT file.
!> Module to create Treelm environment for unit testcases.
module tem_utestEnv_module
  use, intrinsic :: iso_c_binding, only: C_NEW_LINE
  use tem_bc_prop_module,  only: tem_bc_prop_type, load_tem_bc_prop,           &
    &                            tem_empty_bc_prop
  use tem_property_module, only: prp_hasbnd
  use flu_binding,         only: flu_state
  use treelmesh_module,    only: treelmesh_type
  use tem_comm_env_module, only: process
  use aotus_module,        only: open_config_chunk
  use treelmesh_module,    only: load_tem
  use tem_comm_module,     only: tem_commPattern_type
  use tem_aux_module,      only: tem_start
  use tem_general_module,  only: tem_general_load

  implicit none

  character, parameter :: nl = C_NEW_LINE

  character(len=200), parameter :: cubeconf =                                   &
    &   'mesh = {' // nl                                                        &
    & //'         predefined = "cube",' // nl                                   &
    & //'         origin = {0.0, 0.0, 0.0},' // nl                              &
    & //'         length=1.0,' // nl                                            &
    & //'         refinementLevel = 1' // nl                                    &
    & //'       }' // nl

contains

  !> Create the treelm environment for unit tests.
  subroutine load_env(tree, boundary, commPattern )
    !---------------------------------------------------------------------------
    type(treelmesh_type), intent(out) :: tree
    type(tem_bc_prop_type), intent(out) :: boundary 
    type(tem_commPattern_type), intent(out) :: commPattern
    !---------------------------------------------------------------------------
    type(flu_state) :: conf
    integer :: iProp
    !---------------------------------------------------------------------------

    ! Init the Treelm environment
    call tem_start('TREELM unit test')

    print *, "Hello from loadenv" 
    ! Open the configuration file 
    call open_config_chunk(L = conf, chunk = trim(cubeconf))

    call tem_general_load(conf = conf, rank=process%rank, commpattern = commPattern)

    ! Load the mesh first.
    call load_tem(me = tree, conf = conf, myPart = process%rank, &
      &           nParts = process%comm_size )

    ! Load the properties now.
    do iprop = 1, size(tree%Property)
      ! Is the current property setting the prp_hasBnd bit?
      if (tree%global%property(iprop)%bitpos == prp_hasBnd) then
        ! Found the property setting the prp_hasBnd, load the
        ! Boundaries accordingly and leave the loop.
        call load_tem_bc_prop(me = boundary,                &
          &    offset = tree%Property(iprop)%Offset,        &
          &    nElems = tree%Property(iprop)%nElems,        &
          &    basename = trim(tree%global%dirname)//'bnd', &
          &    mypart = process%rank                        )
        exit
      end if
    end do
    

  end subroutine load_env


end module tem_utestEnv_module
