--! [interpolation settings]
interpolation_method = 'linear'
--! [interpolation settings]

-- common header --
--! [reference level]
level =  7  
--! [reference level]
--! [refinement level]
refinementLevel = 1  
--! [refinement level]
label = 'channel'
idLabel = 'channel'
u_in = 0.02
--! [reference length]
length = 2.05
height =  3./16.*length
--! [reference length]
radius = 0.05
usePeriodic = false
useObstacle = true
useCylinder = true
cylOrientation = 'Y'
testIntersection = false

refOmega = 1.9
refLevel = 7

--
m = 2^(refLevel - level)
omega = 1./(1/m*(1/refOmega -0.5)+0.5)
--! [maxlevel]
maxLevel = level+refinementLevel
--! [maxlevel]

dx     = length/math.pow(2,level)
dxMin  = length/math.pow(2,maxLevel)
dxDash = 0.5*dxMin
viscLB = 1/3*(1/omega-1/2)
--! [physical values]
dxPhys = dx
lPhys = length  -- m
csPhys = 300    -- m/s
csLB = 1/math.sqrt(3)
--acoustic scaling with speeds of sound as reference
-- csPhys = csLB * dxPhys / dtPhys
dtPhys = csLB/csPhys*dxPhys
rho0 = 1  -- kg/m^3
viscPhys = viscLB*dxPhys^2/dtPhys
--! [physical values]

--! [iterations]
nElemsMax = 2^maxLevel
nIters = nElemsMax*50
--! [iterations]

-- Use this file as template. Do not modify this file for running some testcases
outputname = 'channel_2D'
folder = 'mesh/' 

-- general refinement levels
-- minrefine: defines minimum refinement level of the entire domain
--
-- maxrefine: defines maximum refinement
-- gets overwritten from madatory STL levels
-- (deprecated - used only for spacer stuff right now)
-- will become default for stl refinement level once
-- it is not mandatory any more
minrefine = 1
maxrefine = level
-- stl_files: for each stl file put a table into
-- this table of tables. Or just a string for
-- the filname, if the other two parameters should
-- get the default: boundary_type = 1, fileformat = 'binary'
-- refinementlevel is the level to which the stl shall be refined
-- it is mandatory

seederLength = length+dx

--! [patch size]
size_x = 0.63*length
start_x = -0.42*length
size_y = height/2-2.*dxDash
start_y = -size_y/2
size_z  = 3*size_y
start_z = 3*start_y
--! [patch size]

-- boundingbox: two entries: origin and length in this
-- order, if no keys are used
--! [bounding box]
boundingbox = {origin = {-seederLength*0.5, -seederLength*.5, -seederLength*0.5},
               length = seederLength}
--! [bounding box]

--! [initial box]
    -- refinebox: three entries: origin, length and refinementlevel
    refinebox = {{origin = {-length*0.5, -height*.5, -0.5*height},
                length = {length, height, height},
                refinementlevel = level
                }
                }
--! [initial box]
--! [refinement box]
    table.insert( refinebox, 
         {origin = {start_x, start_y, start_z }, 
          length = {size_x, size_y, size_z}, 
          refinementlevel = maxLevel
                  }) 
--! [refinement box]

--! [seed definition]
seed = {0.0, 0.0, 0.0 }
--! [seed definition]

--! [geometry walls]
geom={
     plane={
            {vecA={0.0,length,0.0},
              vecB={0.0,0.0,length},
              pos={length*0.5,-length*0.5,-length*0.5},
  	          bclabel='east',
              }
            ,{vecA={0.0,length,0.0},
              vecB={0.0,0.0,length},
              pos={-length*0.5,-length*0.5,-length*0.5},
	            bclabel='west',
              }
             ,{vecA={length,0.0,0.0},
       	      vecB={0.0,0.0,length},
      	      pos={-length*0.5,height*0.5+dxDash,-length*0.5},
	            bclabel='north',
              }
             ,{vecA={length,0.0,0.0},
	            vecB={0.0,0.0,length},
      	      pos={-length*0.5,-height*0.5-dxDash,-length*0.5},
	            bclabel='south',
              }
             
             }
       }
--! [geometry walls]
--! [geometry cylinder]
      stl_files  = {
              { filename = 'cylinder.stl', 
                bclabel = 'obs'},
      }
--! [geometry cylinder]

--! [front back planes]
    table.insert( geom.plane,
         {vecA={length,0.0,0.0},
       	      vecB={0.0,length,0.0},
      	      pos={-length*0.5,-length*0.5,height*0.5+dxDash},
	            bclabel='front',
              })
    table.insert(geom.plane, {vecA={length,0.0,0.0},
	            vecB={0.0,length,0.0},
      	      pos={-length*0.5,-length*0.5,-height*0.5-dxDash},
	            bclabel='back',
              })
--! [front back planes]

