require "seeder"

intp = 'linear'
intp = 'quadratic'
-- inlet_kind = 'wall'
-- outlet_kind = 'wall'
wall_kind = 'wall'
wall_kind = 'wall_linearInterpolation'
inlet_kind='outlet_expol'
outlet_kind='outlet_expol'

scaling='acoustic'
relaxation = 'mrt'
stencil='d3q19'
model='lbm_incomp'
tEnd = 5. 
Re=60.
initChannel = true
testIntp = true
control_routine = true
track = true
interpolation_method = {
  method = intp, 
  --test = testIntp, testEach=true,
  --testFluids = false
  }
if initChannel then
  tMinRamp = 1.
else
  tMinRamp = 0.
end

--require "PATH"
tracking_folder = './tracking/'--"TRACKING_FOLDER"
output_folder = './output/'--"OUTPUT_FOLDER"

-- Simulation name
mesh = './mesh/' -- Mesh information
ebug = {
  erbosity = 100, 
  debugMode = true, 
  debugFiles = true,
  traceMemory = true,
  umpDependencies = true,
  umpTreeIDs = true,
  umpAuxLists = true,
  heckEachStep = true,
  }
simulation_name = simName

identify = {
  label = idLabel, 
  layout=stencil, 
  kind = model,
  relaxation = relaxation}
--! [reference physical values]
u0Phys=1.
viscPhys = u0Phys*heightPhys/Re
csPhys = 300    -- m/s
rho0Phys = 1.
--! [reference physical values]
--! [reference LB values]
csLB = 1/math.sqrt(3)
cs2=1./3.
u0LB=0.05
rho0LB=1.
rho0LB0=0.
omega0=1.7 
--! [reference LB values]
-- determine the relationships 
--! [time step determination]
if scaling == 'acoustic' then
  uLB=u0LB
  dt = uLB/u0Phys*dx
  viscLB = viscPhys*dt/dx/dx
  omega = 1./(3.*viscLB + 0.5)
else
  omega = omega0
  viscLB  = 1./6.*(2./omega - 1.)
  dt = viscLB/viscPhys*dx*dx  
  uLB=u0Phys*dt/dx
end 
--! [time step determination]
-- Reynolds number
Re = heightPhys*u0Phys/viscPhys
ReLB = heightLB*uLB/viscLB
if verbose then
  print('           omega   = '..omega)
  print('          viscLB   = '..viscLB)
  print('        heightLB   = '..heightLB)
  print('             uLB   = '..uLB)
  print('Reynolds number Re = '..ReLB)
  print('          viscPh   = '..viscPhys)
  print('        heightPh   = '..heightPhys)
  print('        lengthPh   = '..length)
  print('             uPh   = '..u0Phys)
  print('Reynolds number Re = '..Re)
end  


amplitude = u_in
--! [reference pressure]
if model == 'lbm_incomp' then
  p0 = 0.
else
  p0 = rho0LB*cs2*dx*dx/dt/dt
end 
--! [reference pressure]

originX = -length*0.25
originY = height*0.
originZ = 00.0
halfwidth = length*0.015
amplitude = 0.00
background = p0
function ic_2Dgauss_pulse(x, y, z)
  return background+amplitude*math.exp(-0.5/(halfwidth^2)*(( x - originX )^2+( y - originY )^2))
end
-- Reference values for the flow state in the 2d stationary channel at laminar
-- flow state
--! [velocity function]
function uX(x,y,z)
    uPhys = u0Phys*(1-(2.*y/heightPhys)^2)
    return uPhys
  end
--! [velocity function]
--! [pressure function]
function pressureRef(x,y,z)
  dp = u0Phys*8.*viscPhys*rho0Phys/heightPhys^2*length
  return p0 + dp*0.5 - dp/length*x
  end
--! [pressure function]
--! [boundary pressure]
pressureIn=pressureRef(-length/2,0.,0.)
pressureOut=pressureRef(length/2,0.,0.)
--! [boundary pressure]
function Sxx(x,y,z)
  return 0.
  end
function Syy(x,y,z)
  return 0.
  end
--! [shear stress function]
function Sxy(x,y,z)
  tauxy= -viscPhys*rho0Phys*8./heightPhys^2*u0Phys*y
  S_xy = tauxy/viscPhys/rho0Phys
  return S_xy
  end
--! [shear stress function]
function stressRef(x,y,z)
    return Sxy(x,y,z)*rho0Phys*viscPhys
  end
  
-- Consistent initial conditions for the channel
if initChannel then
  tRampMin = 1.
  function ic_uX(x,y,z)
    return uX(x,y,z)
    end
  function ic_pressure(x,y,z)
    if( model=='lbm') then
      return pressureRef(x,y,z) + rho0Phys*cs2*dx/dt
    else
      return pressureRef(x,y,z)
    end 
    end
  function ic_Sxx(x,y,z)
    return Sxx(x,y,z)
    end
  function ic_Syy(x,y,z)
    return Syy(x,y,z)
    end
  function ic_Sxy(x,y,z)
    return Sxy(x,y,z)
    end
else
  tRampMin = 0.
  function ic_uX(x,y,z)
    return 0.
    end
  function ic_pressure(x,y,z)
    --return 0.
    return ic_2Dgauss_pulse(x,y,z)
    end
  function ic_Sxx(x,y,z)
    return 0.
    end
  function ic_Syy(x,y,z)
    return 0.
    end
  function ic_Sxy(x,y,z)
    return 0.
    end
end   

init_allElems = false
-- Time step settigs
interval = tEnd/10. 
tRamping = tEnd/10.
--! [time settings]
time = { 
  max = tEnd, 
  interval = interval, 
  useIterations =false,
  wall_max = 3600
}
--! [time settings]
--! [physics table]
physics = { dt = dt, rho0 = rho0Phys, setPhysicalQuantities = true }
--! [physics table]

--! [fluid table]
fluid = { 
       omega = omega, 
       rho0 = rho0LB,
  }
--! [fluid table]

--! [initial conditions]
initial_condition = { pressure  = ic_pressure,
                      velocityX = ic_uX,
                      velocityY = 0.0,
                      velocityZ = 0.0,
                      Sxx = 0.,    
                      Syy = 0.,    
                      Sxy = ic_Sxy 
                      }
--! [initial conditions]

--! [boundary conditions]
boundary_condition = {  
{  label = 'north', 
   kind = 'wall' },
{  label = 'south', 
   kind = 'wall' },
{  label = 'west', 
   kind = inlet_kind,
   pressure= pressureIn},
{  label = 'east', 
   kind = outlet_kind,
   pressure= pressureOut}}
--! [boundary conditions]

if usePeriodic ==false then
  table.insert( boundary_condition, 
{ label = 'frontback', 
   kind = 'wall' } )
  end
if useObstacle ==true then
  table.insert( boundary_condition, 
{ label = 'sphere', 
   kind = wall_kind } )
 end

-- Output settings
utput = { active = true, -- VTK output activated?
  folder = output_folder..label..'_l'..level,     -- Output location
  dumpGhosts = true,
  umpHalos = true,
  time = {min = 0, max = tEnd, interval = interval}
}

add_variable = {
{ name='pressureRef', ncomponents=1, pressureRef=pressureRef },
{ name='stressRef', ncomponents=6, stressRef=stressRef },
}

if track then
-- Tracking              
tracking = {{
  label = 'probePressure', 
 variable = {{'pressure_phy'}},
 shape = {kind = 'canoND', object = {origin ={0.0,0.,0.} } },
 time = {useIterations = true, min = 1, max = -1, interval = 1},
  format = 'ascii', folder = './tracking/'                 
 }, 
  {
  -- tracking object for getting the error in terms of the l2norm
  label = 'hvsXY', 
  variable = { 
     {'pressure_phy'}, 
     {'velocity_phy'}, {'shearstress_phy'}},
  shape = {kind = 'canoND', object = {origin ={ -length*0.5,-height*0.5,0.5*dx}, 
            vec={{length, 0., 0.}, {0.,height,0.}}, 
            segments = {2*nElemsMax, nElemsMax/2} } },
  time = {min = 0., max = tEnd, interval = interval},
  format = 'harvester',
  folder = tracking_folder
 },
  {
  -- tracking object for getting the error in terms of the l2norm
  label = 'hvsPdf', 
  variable = { 
     {'pdf'}
     },
  shape = {kind = 'canoND', object = {origin ={ -length*0.5,-height*0.5,0.5*dx}, 
            vec={{length, 0., 0.}, {0.,height,0.}}, 
            segments = {2*nElemsMax, nElemsMax/2} } },
  time = {min = 0., max = tEnd, interval = interval},
  format = 'harvester',
  folder = tracking_folder
 },
 {
  -- tracking object for getting the error in terms of the l2norm
  label = 'dpdx', 
  variable = { 
     {'pressure_phy'}, {'pressureRef'}
     },
--     {name='velocity', sub_index=1}},
  shape = {kind = 'canoND', object = {origin ={ -length*0.25,-height*0.,0.5*dx}, 
            vec={{length*0.6, 0., 0.}, {0.,0.,0.}}, 
            segments = {2*nElemsMax, nElemsMax/2} } },
  time = {min = tEnd, max = tEnd, interval = interval},
  format = 'asciiSpatial',
  folder = tracking_folder
 }, 
 {
  -- tracking object for getting the error in terms of the l2norm
  label = 'strain', 
  variable = { 
     {'shearstress_phy'}, {'stressRef'}
     },
--     {name='velocity', sub_index=1}},
  shape = {kind = 'canoND', object = {origin ={ -length*0.,-height*0.5,0.5*dx}, 
            vec={{length*0., height, 0.}, {0.,0.,0.}}, 
            segments = {2*nElemsMax, 1} } },
  time = {min = tEnd, max = tEnd, interval = interval},
  format = 'asciiSpatial',
  folder = tracking_folder
 }, 
 {
  -- tracking object for getting the error in terms of the l2norm
  label = '_Errdpdx', 
  variable = { 
     {name='difference', ncomponents=1, dep={'pressure_phy', 'pressureRef'}},
     },
  reduction = {'l2norm'},
--     {name='velocity', sub_index=1}},
  shape = {kind = 'canoND', object = {origin ={ -length*0.25,-height*0.,0.5*dx}, 
            vec={{length*0.6, 0., 0.}, {0.,0.,0.}}, 
            segments = {2*nElemsMax, nElemsMax/2} } },
  time = {min = tEnd, max = tEnd, interval = interval},
  format = 'ascii',
  folder = tracking_folder
 },
 } 
else
  tracking = { }
end

--! [convergence]
table.insert( tracking, 
{ label = 'convergence',
  variable = {{'pressure_phy'}}, 
  shape = {kind = 'canoND', 
    object = {{origin =
         {0.,0.,0.} 
         }}},
  time = {
    interval = interval/20.,
    min = tEnd/2, 
    max = tEnd, 
    },
  folder = './tracking/',
  format='convergence',
  convergence = {
    norm='average', 
    nvals = 100, 
    absolute = false,
    condition = { 
      threshold = 1.e-5, 
      operator = '<=' }
    }
  })
--! [convergence]


