--! [general settings]
simulation_name = 'Gausspulse'

fluid = { omega = 1.7, rho0 = 1.0 }
--! [general settings]



--! [time settings]
time = {
  max = 50,
  interval = 5,
}
--! [time settings]



--! [geometry]
mesh = {
  predefined = 'cube',
  origin = { 0.0, 0.0, 0.0 },
  length = 10.0,
  refinementLevel = 4
}
--! [geometry]



--! [tracking basics]
tracking = {
  label = 'track_pressure',
--! [tracking basics]
--! [tracking vars]
  variable = { {'pressure'}, {'velocity'} },
  folder = './tracking/',
--! [tracking vars]
--! [tracking shape]
  shape = {
    kind = 'canoND',
    object = {origin = {1.0, 1.0, 1.0} }
  },
--! [tracking shape]
--! [tracking format]
  format = 'ascii',
--! [tracking format]
--! [tracking time]
  time = { min = 1, max = -1, interval = 1 }, 
}
--! [tracking time]


--! [ref pressure]
rho0 = 1.
cs2 = 1./3.
p0 = rho0*cs2
--! [ref pressure]

--! [ic function]
function gausspulse(x, y, z)
  originX =  5.0
  halfwidth = 1.0
  amplitude = 0.01
  return p0+amplitude*math.exp(-0.5/(halfwidth^2)*( x - originX )^2)
end
--! [ic function]



--! [initial conditions]
initial_condition = { 
  velocityX = 0.0,
  velocityY = 0.0,
  velocityZ = 0.0,
  pressure  = gausspulse
}
--! [initial conditions]
